﻿Imports Infragistics.Win
Imports Infragistics.Win.UltraWinGrid
Imports System.Globalization

Public Class FlowSheet
    Private Const FS_NONE As Short = 0
    Private Const FS_NORMAL As Short = 1
    Private Const FS_FOLLOWUP As Short = 2
    Private Const FS_LIST As Short = 3

    Private Const LVL_GROUP As Integer = 1                 ' Indent levels for flow sheet
    Private Const LVL_BODYSYS As Integer = 2
    Private Const LVL_FINDING As Integer = 3

    Private Const COL_GROUP As Integer = 0
    Private Const COL_SUBGROUP As Integer = 1
    Private Const COL_HEAD As Integer = 2
    Private Const COL_POS As Integer = 3                   ' column indices for fsList
    Private Const COL_NEG As Integer = 4
    Private Const COL_FINDING As Integer = 5
    Private Const COL_OFFSET As Integer = 6

    Private mintImgNegOff As Short = 0
    Private mintImgNegOn As Short = 1
    Private mintImgPosOff As Short = 2
    Private mintImgPosOn As Short = 3

    Private myCurrentMedcinid As Integer            ' Medcinid for current add/edit
    Private myFollowUpColumn As Integer             ' Number of a prompt column, or 0 if none

    Private mstrDateFm As String = ""               ' Medcin date format strings used for history pool filtering
    Private mstrDateTo As String = ""
    Private mstrTimeFm As String = ""
    Private mstrTimeTo As String = ""
    Private mstrCaption As String                   ' Caption of current FlowSheet
    Private mstrEncounterTime As String             ' Medcin string version of etime
    Private mstrEncounterList As String             ' List of encounter times, for printing
    Private mstrFlowSheetList As String             ' To preserve the list for changing listsize
    Private mstrFlowSheetBase As String             ' To preserve 1 or more base Medcinids
    Private mstrHistoryList As String               ' List of encounters in history pool
    Private mstrPatientName As String
    Private mstrNegativeText As String = "N"        ' Display for "N"
    Private mstrPositiveText As String = "P"        ' Display for "P"
    Private mstrDayFormat As String                 ' User's time format preference for day precision

    Private mintFlowGroup As Short           ' Current group number while building flow sheet
    Private mintFlowSubGroup As Short               ' Current subgroup number while building flow sheet
    Private myFlowSheetMode As Short = FS_NORMAL    ' Mode for view

    Private mbolHaveMedcinData As Boolean = False
    Private mbolHaveHistoryPool As Boolean

    Private mbolAllowEdit As Boolean                ' True if encounter is editable
    Private mbolAllowRowMenu As Boolean = True      ' True if container is allowing our row menu
    Private mbolCiteMode As Boolean                 ' True if in auto cite on click
    Private mbolLoading As Boolean
    Private mbolSwapNeg As Boolean = False
    Private myUseCellDataButton As Boolean = True
    Private mbolLoadDirect As Boolean = False
    Private mbolShowInsertButton As Boolean = False
    Private mbolUnitsInHeader As Boolean = False    ' Whether or not units when same in row are displayed in row header

    Private myHeaderRowBackColor As System.Drawing.Color = Color.LightBlue
    Private myHeaderRowForeColor As System.Drawing.Color = Color.Black
    Private myGroupRowBackColor As System.Drawing.Color = Color.White
    Private myGroupRowForeColor As System.Drawing.Color = Color.Black
    Private mySystemRowBackColor As System.Drawing.Color = Color.White
    Private mySystemRowForeColor As System.Drawing.Color = Color.Black
    Private myFindingRowBackColor As System.Drawing.Color = Color.White
    Private myFindingDescriptionForeColor As System.Drawing.Color = Color.Black
    Private myFollowUpDescriptionForeColor As System.Drawing.Color = Color.Teal
    Private myPositiveTextForeColor As System.Drawing.Color = Color.Red
    Private myNegativeTextForeColor As System.Drawing.Color = Color.Blue
    Private myCriticalTextBackColor As System.Drawing.Color = Color.Yellow
    Private myCriticalTextForeColor As System.Drawing.Color = Color.Red

    Private myHeaderActions As ColumnHeaderActions = ColumnHeaderActions.AllowCite Or ColumnHeaderActions.AllowNarration _
     Or ColumnHeaderActions.AllowHeaderMenu

    Private dbTable As DataTable = Nothing
    Private myMouse As System.Windows.Forms.MouseEventArgs = Nothing
    Private myDrawFilter As CellDrawFilter = New CellDrawFilter
    Private myEventRow As UltraGridRow = Nothing                  ' Row of the current flow sheet cell

    Private Const STYLE_POS As String = "MyPos"
    Private Const STYLE_NEG As String = "MyNeg"
    Private Const STYLE_HEAD As String = "MyHead"
    Private Const STYLE_HEADER As String = "MyHeader"
    Private Const STYLE_GROUP As String = "MyGroup"
    Private Const STYLE_SYSTEM As String = "MySystem"
    Private Const STYLE_BORDER As String = "MyBorder"
    Private Const STYLE_CRITICAL As String = "MyCritical"
    Private Const STYLE_FOLLOWUP As String = "MyFollowup"

    Private gFolderPath As String = Application.ExecutablePath
    Private Const STR_LBS As String = "lbs"
    ' This is used for passing finding lists to fQuick
    Private mcolQuickData As Collection

    ''' <summary>
    ''' This structure contains one item of a list. Lists of items
    ''' are returned as a System.ArrayList.
    ''' </summary>
    Public Structure ResponseItem
        ''' <summary>
        ''' This field contains the unique identifier for the finding.
        ''' </summary>
        Public Medcinid As Integer
        ''' <summary>
        ''' This field typically contains the suggested prefix for the finding.
        ''' </summary>
        Public Prefix As String
        ''' <summary>
        ''' This field contains the nodekey of the finding.
        ''' </summary>
        Public Nodekey As String
        ''' <summary>
        ''' The Description field is the description of the medcinid.
        ''' </summary>
        Public Description As String
        ''' <summary>
        ''' This field contains the terminology type of the finding.
        ''' </summary>
        Public TermType As Short
        ''' <summary>
        ''' This field typically contains the property flags value.
        ''' </summary>
        Public Flags As Short
        ''' <summary>
        ''' The Subs field, if '+', indicates that the medcinid has children.
        ''' </summary>
        Public Subs As String
        ''' <summary>
        ''' The Dx field is set to non-zero if the medcinid refers to a computed diagnosis 
        ''' from which the intelligent prompting lists are derived from.
        ''' </summary>
        Public Dx As Short
    End Structure

#Region "Hacks"
    Private Sub RaiseFindingAction(ByVal mode As FindingActionMode, ByVal finding As cFlowFinding)
        Select Case mode
            Case FindingActionMode.fscAddFinding
                finding.ClientKey = "K" & finding.Medcinid
            Case FindingActionMode.fscDeleteFinding
                finding.ClientKey = String.Empty
            Case FindingActionMode.fscEditFinding
                ' Hack the result w/o checking for inverse normal
                ' Set the result according to the button clicked
                If (finding.ButtonState = FindingButtonState.fscButtonNegative) Then
                    finding.Chart.Result = RESULT_NEG
                Else
                    finding.Chart.Result = RESULT_BLANK
                End If
            Case Else
        End Select
    End Sub

    Private Function RetrieveQualifierCaption(ByVal type As String, ByVal Code As String) As String
        Dim rc As String = "Unknown"    'String.Empty
        Return rc
    End Function

    Private Sub ShowRowContextMenu(ByVal Row As UltraGridRow, ByVal Finding As cFlowFinding)

    End Sub

#End Region

#Region "Enums"

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This enumerates the actions that can occur on a finding row.
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Enum FindingActionMode
        ''' <summary>
        ''' No action
        ''' </summary>
        fscNoAction = 0
        ''' <summary>
        ''' The user has clicked the unlit button of a current grid cell that does not 
        ''' have a finding associated with it, or the user is citing a finding . 
        ''' This action is interpreted as a request to add the finding to the list.
        ''' </summary>
        fscAddFinding = 1
        ''' <summary>
        ''' The user has clicked the unlit button of a current grid cell that has a 
        ''' finding associated with it. This action is interpreted as a request to 
        ''' change the finding in the list from positive to negative or vice versa.
        ''' </summary>
        fscEditFinding = 2
        ''' <summary>
        ''' The user has clicked a lit button of a current grid cell that has a finding 
        ''' associated with it. This action is interpreted as a request to delete the 
        ''' finding from the list. If your application deletes the finding, the ClientKey 
        ''' property of the finding object should be cleared.
        ''' </summary>
        fscDeleteFinding = 3
        ''' <summary>
        ''' The user has selected a grid row with a finding associated with the current 
        ''' column grid cell. This action is interpreted as a request to select this 
        ''' finding from the list for editing.
        ''' </summary>
        fscSelectFinding = 4
        ''' <summary>
        ''' The user has selected a grid row that does not have a finding associated 
        ''' with the current encounter column. This action is interpreted as a request 
        ''' to make that fnding the current finding.
        ''' </summary>
        fscSetCurrentFinding = 5
        ''' <summary>
        ''' Specifies that the container should look for the finding in its encounter.
        ''' </summary>
        fscQueryFindingInChart = 6
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Enumerates the states of the Pos|Neg buttons on the current column
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Enum FindingButtonState
        ''' <summary>No button is active.</summary>
        fscButtonNone = 0
        ''' <summary>The first button is active.</summary>
        fscButtonPositive = 1
        ''' <summary>The second button is active.</summary>
        fscButtonNegative = 2
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Enumerates options for the time search dialog
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Enum tsOptions As Short
        ''' <summary>The 'All Time Periods' option.</summary>
        All = 1
        ''' <summary>The 'Last N' option.</summary>
        LastN = 2
        ''' <summary>The 'Sliding Time Period" option.</summary>
        Sliding = 3
        ''' <summary>The 'Specific Time Period' option.</summary>
        Specific = 4
        ''' <summary>The 'Time' option.</summary>
        Time = 5
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Enumerates the flow sheet header row context menu items
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    <FlagsAttribute()> _
    Public Enum HeaderMenuItems As Integer
        ''' <summary>Show the Time Search dialog.</summary>
        TimeSearch = 1
        ''' <summary>Specify column order ascending or descinding.</summary>
        ColumnOrder = 2
        ''' <summary>Specify column precision: Year, Month, Week, Day, Hour, Minute.</summary>
        ColumnPrecision = 4
        ''' <summary>Show or hide empty data rows.</summary>
        ShowEmptyRows = 8
        ''' <summary>Include free text findings in dynamic flow sheets.</summary>
        IncludeFreeText = 16
        ''' <summary>Show or hide the information bar.</summary>
        ShowInfoBar = 32
        ''' <summary>Toggles the Header Date presentation between day and variable.</summary>
        ToggleHeaderPrecision = 64
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Enumerates the flow sheet data rows context menu items
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    <FlagsAttribute()> _
    Public Enum RowMenuItems As Integer
        ''' <summary>Raise the event to show a reference for the row finding.</summary>
        ShowReference = 1
        ''' <summary>Hide all selected non-empty rows.</summary>
        HideRows = 2
        ''' <summary>Unhide all selected non-empty rows.</summary>
        UnhideRows = 4
        ''' <summary>Insert a row.</summary>
        InsertRow = 8
        ''' <summary>Raise an event to graph row(s).</summary>
        GraphRows = 16
        ''' <summary>Show or hide children rows with data.</summary>
        ShowChildren = 32
        ''' <summary>Unhide all non-empty rows.</summary>
        UnhideAllRows = 64
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Enumerates the available cell actions
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Enum CellActions As Integer
        ''' <summary>No tool tip will be shown when hovering over a data cell.</summary>
        ToolTipNone = 0
        ''' <summary>The BeforeCellToolTip event data contain only identity information.</summary>
        ToolTipApplication = 1
        ''' <summary>A tool tip consisting of the cell data will be shown when hovering over a data cell.</summary>
        ToolTipCellData = 2
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This enumerates the available column header actions
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    <FlagsAttribute()> _
    Public Enum ColumnHeaderActions As Integer
        ''' <summary>A tool tip will not be shown when hovering.</summary>
        ToolTipNone = 0
        ''' <summary>A tool tip consisting of the header date will be shown when hovering.</summary>
        ToolTipHeaderDate = 1
        ''' <summary>A tool tip consisting of the encounter times included in the column will be shown when hovering.</summary>
        ToolTipColumnDates = 2
        ''' <summary>A left mouse click on the header column will show a narration dialog.</summary>
        AllowNarration = 4
        ''' <summary>A left mouse click on the header column will cite the column when cite mode is enabled.</summary>
        AllowCite = 8
        ''' <summary>A right mouse click on the header column will show a context menu.</summary>
        AllowHeaderMenu = 16
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This enumerates the possible ways of using precision to format the encounter
    ''' date and time used in the column header.
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	10/24/2006	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Enum DateHeaderType As Short
        ''' <summary>The header date string will be formated using Day precision.</summary>
        FixedDayPrecision = 0
        ''' <summary>The header date string will be formated using the column precision.</summary>
        UseColumnPrecision = 1
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Enumerates the type of a grid row
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Enum GridRowType As Integer
        ''' <summary>The row is the header row.</summary>
        Header = 1
        ''' <summary>The row is a group row.</summary>
        Group = 2
        ''' <summary>The row is a subgroup row.</summary>
        SubGroup = 4
        ''' <summary>The row is a finding data row.</summary>
        Data = 8
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This enumerates the state of a grid row
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	6/16/2006	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    <FlagsAttribute()> _
    Public Enum RowState As Short
        ''' <summary>The Row is normal; not collapsed and not hidden.</summary>
        Normal = 0
        ''' <summary>The row is hidden.</summary>
        Hidden = 1
        ''' <summary>The row is collapsed.</summary>
        Collapsed = 2
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Enumerates the possible cell styles used for text color
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Private Enum CellDisplayStyle As Short
        ''' <summary>The cell display style is a negative finding.</summary>
        Negative = 0
        ''' <summary>The cell display style is a positive finding.</summary>
        Positive = 1
        ''' <summary>The cell display style is a critical finding.</summary>
        Critical = 2
    End Enum

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This enumerates the possible actions for handling unit strings
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	10/18/2006	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Private Enum RowUnitAction As Short
        ''' <summary>Units should be removed from cells and added to row header.</summary>
        RemoveFromCells = 0
        ''' <summary>Units should be removed from row header and added to cells.</summary>
        AddToCells = 1
    End Enum
#End Region

    Public Sub PostError(ByVal ex As Exception)
        MsgBox(ex.Message)
    End Sub

    Public Sub PostMessage(ByVal MsgNumber As Integer, ByVal MsgString As String)
        MsgBox(MsgString, MsgBoxStyle.Information, Application.ProductName)
    End Sub


    Public Sub ImportGrid()
        Dim DlgFile As OpenFileDialog

        Try
            DlgFile = New OpenFileDialog
            DlgFile.InitialDirectory = gFolderPath
            DlgFile.Filter = "txt files (*.txt)|*.txt"
            DlgFile.FilterIndex = 1
            If DlgFile.ShowDialog() = DialogResult.OK Then
                Dim FilePath As String = DlgFile.FileName
                gFolderPath = System.IO.Path.GetDirectoryName(FilePath)
                ProcessBuildListDirect(ImportGridData(FilePath))
            End If
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    Private Function ImportGridData(ByVal FilePath As String) As ArrayList
        Dim arList As New ArrayList
        Dim oItem As ResponseItem
        Dim bFileOpen As Boolean

        Try
            FileOpen(2, FilePath, OpenMode.Input)
            bFileOpen = True
            Do While Not EOF(2)
                oItem = New ResponseItem
                With oItem
                    Input(2, .Medcinid)
                    Input(2, .Nodekey)
                    Input(2, .Description)
                    Input(2, .Prefix)
                    Input(2, .Subs)
                    Input(2, .TermType)
                    Input(2, .Flags)
                    Input(2, .Dx)
                End With
                arList.Add(oItem)
                oItem = Nothing
            Loop
            bFileOpen = False
            FileClose(2)
        Catch ex As Exception
            PostError(ex)
        End Try
TheExit:
        If (bFileOpen) Then
            bFileOpen = False
            FileClose(2)
        End If
        oItem = Nothing
        Return arList
    End Function


    Private Sub btnLoadGridDirect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLoadGridDirect.Click
        mbolLoadDirect = True
        ImportGrid()
        mbolCiteMode = True   ' Use to test out fQuick
        mbolAllowEdit = True
    End Sub


    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        myFindingDescriptionForeColor = System.Drawing.SystemColors.ControlText
        myFindingRowBackColor = System.Drawing.Color.White
        myFollowUpDescriptionForeColor = System.Drawing.SystemColors.ControlText
        myGroupRowBackColor = System.Drawing.Color.Aqua
        myGroupRowForeColor = System.Drawing.Color.Blue
        mySystemRowBackColor = Color.Bisque
        'myHeaderRowBackColor = System.Drawing.SystemColors.GradientInactiveCaption
        myHeaderRowForeColor = System.Drawing.Color.Black
        mstrEncounterTime = Date2Mdate(Now)
        InitializeGrid()
    End Sub

    Private Function Date2Mdate(ByVal dt As Date) As String
        ' Procedure converts a date to a Medcin date format
        Date2Mdate = Format(dt, "yyyyMMddHHmmss")
    End Function

    ''' <summary>
    ''' Procedure to add a flow sheet finding to the current encounter
    ''' </summary>
    ''' <param name="Row">The UltraGridRow of the selected finding.</param>
    ''' <param name="Col">The column index of the selected finding.</param>
    ''' <remarks></remarks>
    Private Sub AddFlowFindingToEncounter(ByVal Row As UltraGridRow, ByVal Col As Integer)
        Dim strCellData(), strField, strData As String
        Dim strCiteText As String
        Dim Index, Medcinid As Integer
        Dim Flag As Short
        Dim bolPositive, bolReverse As Boolean
        Dim objFinding As cFlowFinding
        Dim objChart As MedChart
        Dim oRowData As cFlowRowData
        Dim oCell As cCellData
        Dim DisplayStyle As CellDisplayStyle

        Try

            objFinding = New cFlowFinding
            objChart = New MedChart
            With ugFlowSheet
                ' Not using this in the flow sheet control
                '' Check if need popup menu to confirm action
                ''If ((strEtime <> mstrLastETime Or mintCiteAsOf < 2) And myMouseButton = vbRightButton) Then
                ''    mnuCell(1).Caption = "Cite As Of " & strEtime
                ''    mintCiteAsOf = -1   ' Set to detect escape from menu
                ''    PopupMenu(popUpCellMenu, vbPopupMenuRightButton)
                ''    If (mintCiteAsOf = -1) Then Exit Sub
                ''    mstrLastETime = strEtime
                ''End If

                ' Medcinid stored in rowData CFlowRowData object
                oRowData = Row.Tag
                Medcinid = oRowData.Medcinid
                Flag = oRowData.Flag
                ' Cell:
                ' "Column Number\tSequence Number"
                ' May include extended data:
                ' "\tv=value\tu=unit\to=onset\td=duration\tn=note\tr=rxdata"
                oCell = Row.Cells(Col).Tag
                strCiteText = Row.Cells(Col).Value
                ' If the cell contains more than 1 finding, show dialog for choice
                If (oCell.CellData.Count > 1) Then
                    ' Use this code when Infragistics grid is in the project
                    'Dim DlgList As New fFSList
                    'DlgList.SetList(strCiteText, oCell.CellData)
                    'If (DlgList.ShowDialog(Me) = DialogResult.OK) Then
                    '    strData = oCell.CellData(DlgList.getChoice).ExtendedData
                    '    strCiteText = DlgList.getItem
                    'Else
                    '    Exit Sub
                    'End If
                    ' Hack in single value for now
                    strData = CType(oCell.CellData(0), cCellFinding).ExtendedData
                Else
                    ' Only one finding in the cell, use it
                    strData = CType(oCell.CellData(0), cCellFinding).ExtendedData
                End If
            End With
            With objChart
                ' If the cited text is simply P or N, then
                ' the cell may not have any cpData;
                ' Use the cell text to determine result
                .Medcinid = Medcinid
                strCellData = strData.Split(vbTab)
                ' If the cited cell has a "P", then its positive sense
                DisplayStyle = CellDisplayStyle.Negative
                If (strCiteText = mstrPositiveText) Then
                    DisplayStyle = CellDisplayStyle.Positive
                    bolPositive = True
                    ' Otherwise, if the cell is not "N", check extended data
                ElseIf (strCiteText <> mstrNegativeText) Then
                    If (UBound(strCellData) > 1) Then
                        If (RetrieveField(strData, "q=") = "+") Then
                            bolPositive = True
                            DisplayStyle = CellDisplayStyle.Positive
                        End If
                    End If
                End If
                bolReverse = False
                ' Set the result based on positive|negative
                If (bolPositive) Then
                    If (bolReverse) Then
                        .Result = RESULT_ABN
                    End If
                Else
                    If (bolReverse) Then
                        .Result = RESULT_BLANK
                    Else
                        .Result = RESULT_NEG
                    End If
                End If
                If (UBound(strCellData) > 1) Then ' Extra data
                    Dim UniqueID As Integer
                    ' Unpack the extra data into chart fields
                    For Index = 2 To UBound(strCellData)
                        strField = strCellData(Index)
                        If (strField.Length > 2) Then
                            strData = strField.Substring(2)
                            Select Case strField.Substring(0, 1)
                                Case "s" : UniqueID = CInt(strData)
                                    'Case "e" : strEtime = strData
                                Case "p"
                                    If (strData = "") Then strData = PREFIX_BLANK
                                    .Prefix = strData
                                Case "m" : .Modifier = strData
                                Case "t"                ' Use result from above
                                Case "a" : .Status = strData
                                Case "c" : .Flag = strData
                                Case "v"
                                    .Value = strData
                                    strCiteText = strData
                                Case "y" : .RangeNormalLow = Val(strData)
                                Case "z" : .RangeNormalHigh = Val(strData)
                                Case "L" : .RangeScale = CShort(strData)
                                Case "u" : .Unit = strData
                                Case "o" : .Onset = strData
                                Case "d" : .Duration = strData
                                Case "k" : .Episode = strData
                                Case "n"
                                    '' The note is chopped for encounter history to a max of 48 chars
                                    '' If what we grabbed was exactly 48, retrieve entire string from server
                                    'If (strData.Length = 48) Then
                                    '    If (UniqueID > 0) Then
                                    '        strField = MedClient.HistGetItemData(UniqueID, "n")
                                    '        If (strField.Length > 2) Then
                                    '            strData = strField.Substring(2)
                                    '        End If
                                    '    End If
                                    'End If
                                    .Note = strData
                                Case "r"              ' Do not know what to do with this
                                Case "b" : .Referid = strData
                                Case Else
                            End Select
                        End If
                    Next
                End If
                If (.Value.Length > 0 And .Unit.Length > 0) Then
                    ' Add the units only if they are not supposed to be in the header
                    If (Not (mbolUnitsInHeader And oRowData.UnitsMatch)) Then
                        strCiteText = strCiteText & " " & .Unit
                    End If
                    If (.RangeScale = 1 OrElse .RangeScale = 7) Then
                        DisplayStyle = CellDisplayStyle.Critical
                    End If
                End If
                'If (omdMouse.Button = MouseButtons.Right And mintCiteAsOf >= 1) Then
                '    .Onset = strEtime
                'End If
            End With

            With objFinding
                .Medcinid = Medcinid
                .Group = oRowData.Group
                .Chart = objChart
                If (mbolSwapNeg And bolReverse) Then
                    .ButtonState = IIf(bolPositive, FindingButtonState.fscButtonNegative, FindingButtonState.fscButtonPositive)
                Else
                    .ButtonState = IIf(bolPositive, FindingButtonState.fscButtonPositive, FindingButtonState.fscButtonNegative)
                End If
            End With
            RaiseFindingAction(FindingActionMode.fscAddFinding, objFinding)

            If (objFinding.ClientKey.Length > 0) Then
                Dim ugCell As UltraGridCell = Row.Cells(COL_FINDING)
                Dim Style As String = ""
                ' Add was successful, set up the current encounter cell
                With ugFlowSheet
                    .ActiveCell = ugCell                     ' Column COL_FIND is the current encounter
                    ' The color should be the same as that from the cited cell
                    ' We need to use a style for this!
                    Select Case DisplayStyle
                        Case CellDisplayStyle.Critical
                            ugCell.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_CRITICAL)
                        Case CellDisplayStyle.Negative
                            ugCell.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_NEG)
                        Case CellDisplayStyle.Negative : Style = STYLE_POS
                            ugCell.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_POS)
                    End Select
                    ugCell.Value = strCiteText
                    ' Make the cFlowFinding object the link for the current encounter cell
                    ugCell.Tag = objFinding
                    AddIconsToPosNeg(Row, objFinding.ButtonState, (mbolSwapNeg And bolReverse))
                End With
            End If
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    ''' <summary>
    ''' Procedure to populate the positive and negative column button images
    ''' </summary>
    ''' <param name="Row">The UltraGridRow of the flow sheet finding</param>
    ''' <param name="State">The state for setting the images</param>
    ''' <param name="Reverse">If reverse normal is applicable for the finding</param>
    ''' <remarks></remarks>
    Private Sub AddIconsToPosNeg(ByVal Row As UltraGridRow, ByVal State As FindingButtonState, ByVal Reverse As Boolean)
        ' Procedure adds the icons to a list row
        Dim iPos, iNeg As Integer
        ' Guard for wrong row
        If (Not Row.IsDataRow) Then Exit Sub
        Select Case State
            Case FindingButtonState.fscButtonPositive
                If (Reverse) Then
                    iPos = mintImgNegOn
                    iNeg = mintImgPosOff
                Else
                    iPos = mintImgPosOn
                    iNeg = mintImgNegOff
                End If
            Case FindingButtonState.fscButtonNegative
                If (Reverse) Then
                    iPos = mintImgNegOff
                    iNeg = mintImgPosOn
                Else
                    iPos = mintImgPosOff
                    iNeg = mintImgNegOn
                End If
            Case Else
                If (Reverse) Then
                    iPos = mintImgNegOff
                    iNeg = mintImgPosOff
                Else
                    iPos = mintImgPosOff
                    iNeg = mintImgNegOff
                End If
        End Select
        ' Add the cell button images
        Try
            If (Row.Cells.Count > COL_NEG) Then
                Dim cell As UltraGridCell = Row.Cells(COL_POS)
                cell.Appearance.Image = imgBitMaps.Images(iPos)
                ' Store the current state in the data property of COL_POS
                cell.Tag = State
                cell = Row.Cells(COL_NEG)
                cell.Appearance.Image = imgBitMaps.Images(iNeg)
            End If
        Catch ex As Exception

        End Try
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Procedure takes ResponseList data in spreadsheet mode to construct the flow sheet view
    ''' </summary>
    ''' <param name="Medcinid"></param>
    ''' <param name="Nodekey"></param>
    ''' <param name="Description"></param>
    ''' <param name="ExtendedData">This is the ResponseItem's Prefix field.</param>
    ''' <param name="Subs"></param>
    ''' <param name="Termtype"></param>
    ''' <param name="Flags"></param>
    ''' <param name="LevelOne">Not using this for Infragistics grid</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	6/6/2006	Modified for 3Q2006 flowsheet enhancements
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Private Sub AddRecordToFlowSheet(ByVal Medcinid As Integer, ByVal Nodekey As String, ByVal Description As String, ByVal ExtendedData As String, ByVal Subs As String, ByVal Termtype As Short, ByVal Flags As Short, ByVal LevelOne As Short)
        Dim Col, Index As Integer
        Dim iSeq As Integer
        Dim strPosneg As String = Nothing
        Dim strCell, strEncTime As String
        Dim strDates(), strCellData(), strField As String
        Dim strHeaders() As String = Nothing
        Dim strPrefixCode As String
        Static strGroup As String
        Static strSubGroup As String
        Dim Style As String = ""
        Static oRowData As cFlowRowData
        Dim bolHaveHeaders As Boolean = False
        Dim alColData As ArrayList
        Dim dbCol As DataColumn
        Static ugRow As UltraGridRow


        Try
            '  Debug.Print Medcinid, , Nodekey, Flags, Subs, ExtendedData, LevelOne, Description
            ' Determine what kind of record this is
            Select Case Medcinid
                Case -1                           ' List of etimes in Description & ExtendedData
                    Dim strFollowupCaption As String = String.Empty
                    Select Case myFlowSheetMode
                        Case FS_NORMAL : strFollowupCaption = "Prompt"
                        Case FS_FOLLOWUP : strFollowupCaption = "Follow up"
                        Case Else : strFollowupCaption = "Empty Row"
                    End Select
                    dbTable = New DataTable         ' Create a new table for the data
                    With dbTable
                        ' Add the fixed columns
                        dbCol = New DataColumn("Group", GetType(String))
                        .Columns.Add(dbCol)
                        dbCol = New DataColumn("SubGroup", GetType(String))
                        .Columns.Add(dbCol)
                        dbCol = New DataColumn("Head", GetType(String))
                        .Columns.Add(dbCol)
                        dbCol = New DataColumn("Pos", GetType(String))
                        .Columns.Add(dbCol)
                        dbCol = New DataColumn("Neg", GetType(String))
                        .Columns.Add(dbCol)
                        dbCol = New DataColumn("Finding", GetType(String))
                        .Columns.Add(dbCol)
                    End With

                    If (Nodekey.Length > 0) Then    ' Server handed us the header info
                        ParseHdrInfo(Nodekey)
                    End If
                    myFollowUpColumn = 0                            ' Init to no f/u column
                    If (Description.Length = 0 Or Description.Equals("0")) Then
                        mstrEncounterList = ""                  ' The list is empty
                        '    .Cols.Count = 5
                        '    .Cols.Frozen = 4                         ' Don't scroll header or current enc
                        myFollowUpColumn = COL_OFFSET
                        '    .Item(0, myFollowUpColumn) = strField
                        '    .Cols(COL_FINDING + 1).UserData = ""
                        ugFlowSheet.DataSource = dbTable
                    Else
                        mstrEncounterList = Description
                        strDates = Description.Split(vbTab)
                        If (ExtendedData.Length > 0) Then
                            strHeaders = ExtendedData.Split(vbTab)
                            bolHaveHeaders = True
                        End If
                        ' One col for headings
                        '    lngWidth = fsFlowSheet.Width - 40
                        ' Apply an offset to the servers
                        ' list of etimes three columns to the right as we
                        ' will load the current encounter in columns 1, 2, and 3
                        ' Upper bound + 1 for count, and we're using 4 cols
                        '.Cols.Count = UBound(strDates) + 5
                        'If (.Cols.Count > 4) Then
                        '    .Cols.Frozen = 4                     ' Don't scroll header or current enc
                        'End If
                        For Index = 0 To UBound(strDates)
                            strEncTime = strDates(Index)
                            ' If the strEncTime is a null string, then this is a prompt or follow up data column
                            If (strEncTime.Length = 0 OrElse strEncTime = "0") Then
                                strField = strFollowupCaption
                                ' Need this for testing later
                                myFollowUpColumn = Index + COL_OFFSET
                            Else
                                If (bolHaveHeaders) Then
                                    strField = strHeaders(Index)
                                Else
                                    strField = FormatHeaderTime(strEncTime)
                                End If
                            End If
                            dbCol = New DataColumn(strField)
                            dbTable.Columns.Add(dbCol)
                        Next
                        ugFlowSheet.DataSource = dbTable
                        With ugFlowSheet.DisplayLayout.Bands(0)
                            For Index = 0 To UBound(strDates)
                                strEncTime = strDates(Index)
                                ' Store the etime string for the column in column data.
                                ' The 1st element will be the header etime. Succeeding
                                ' elements will be the unique etimes of the column.
                                alColData = New ArrayList
                                alColData.Add(strEncTime)       ' Put the Medcin date format header in band's Column Tag
                                .Columns(Index + COL_OFFSET).Tag = alColData
                            Next
                        End With
                    End If
                    With ugFlowSheet.DisplayLayout.Bands(0).Columns(COL_FINDING)
                        .Header.Caption = FormatHeaderTime(mstrEncounterTime)
                        .Tag = mstrEncounterTime
                    End With
                    If (myFollowUpColumn > 0) Then
                        ugFlowSheet.DisplayLayout.Bands(0).Columns(myFollowUpColumn).Header.Caption = strFollowupCaption
                    End If

                    '' Apply preferences
                    'With fsFlowSheet
                    '    '.Cell(flexcpFont, 0, 0, 0, fsFlowSheet.Cols - 1) = mfntFontHeader
                    '    .Styles.Fixed.ForeColor = myHeaderRowForeColor
                    '    .Styles.Fixed.BackColor = myHeaderRowBackColor
                    '    ' Add the vertical grid lines only in the non-frozen finding columns
                    '    '.Select(0, 4, 0, fsFlowSheet.Cols - 1)
                    '    '.CellBorder(vb3DDKShadow, -1, -1, -1, -1, 1, -1)
                    '    Range = .GetCellRange(0, 4, 0, fsFlowSheet.Cols.Count - 1)
                    '    Range.Style = .Styles(STYLE_HEADER)
                    'End With

                Case -2                             ' Group header: description has string
                    strGroup = Description
                    mintFlowGroup = Termtype        ' Save the group for setting CFlowRowData

                Case -3                             ' Body system header: description has string
                    strSubGroup = Description
                    mintFlowSubGroup = Termtype        ' Save the subgroup for setting CFlowRowData

                Case Else                         ' A finding
                    Dim bOtherData As Boolean
                    Dim strEtime As String
                    Dim strValue As String = String.Empty
                    Dim strUnit As String = String.Empty
                    Dim strNote As String = String.Empty
                    Dim RangeScale As Short = 0
                    Dim DisplayStyle As CellDisplayStyle
                    Dim oCell As cCellData = Nothing
                    Dim oFinding As cCellFinding

                    ' Findings are fired across the grid; the first one has the description.
                    If (Description.Length > 0) Then
                        ' Keep the Medcinid, Nodekey, Group number, and SubGroup number
                        oRowData = New cFlowRowData(Medcinid, Nodekey, Description, mintFlowGroup, mintFlowSubGroup)
                        oRowData.Prefix = RetrieveField(ExtendedData, "p=") ' and prefix for the description
                        If (oRowData.Prefix = "") Then oRowData.Prefix = PREFIX_BLANK
                        oRowData.Flag = Flags   ' And the property flag; Supplied by server in Buildlist starting Feb 2004
                        ' Create a new grid row
                        ugRow = ugFlowSheet.DisplayLayout.Bands(0).AddNew()
                        With ugRow
                            .Cells(COL_GROUP).Value = strGroup
                            .Cells(COL_GROUP).Tag = mintFlowGroup           ' Put the group number in the Tag
                            .Cells(COL_SUBGROUP).Value = strSubGroup
                            .Cells(COL_SUBGROUP).Tag = mintFlowSubGroup     ' Put the SubGroup number in the Tag
                            .Cells(COL_HEAD).Value = Description
                            .Tag = oRowData ' Store a reference to the oRowData object in the row's Tag property
                        End With

                        ''If (LevelOne > 0 And Subs = "+") Then
                        'If (LevelOne > 0) Then
                        '    ' Row has children findings; make it a node
                        '    With .Rows(NewRow)
                        '        .IsNode = True
                        '        .Node.Level = LVL_FINDING
                        '    End With
                        'End If
                        ' Add the Pos and Neg buttons
                        AddIconsToPosNeg(ugRow, FindingButtonState.fscButtonNone, False)
                        '' Add the vertical grid lines only in the non-frozen finding columns
                        'Range = .GetCellRange(NewRow, 3, NewRow, fsFlowSheet.Cols.Count - 1)
                        'Range.Style = .Styles(STYLE_BORDER)
                    Else
                        ' The row to use is the last row which is static ugRow
                    End If
                    oFinding = New cCellFinding(ExtendedData)       ' Create a cCellFinding object
                    ' ExtendedData:
                    ' "Column Number\tSequence Number"
                    ' May include add'l data:
                    ' "\te=etime\tq=+|-\tv=value\tu=unit\to=onset\td=duration\tn=note\tr=rxdata"
                    strCellData = Split(ExtendedData, vbTab, -1)
                    strPrefixCode = ""
                    ' Column numbers from server are:
                    '   0 for caption, 1, 2, 3, ... for data columns;
                    '   so data col 1 is our col 4
                    Col = Val(strCellData(0)) + COL_FINDING    ' Get the grid column
                    Dim ugCell As UltraGridCell = ugRow.Cells(Col)
                    If (Col = myFollowUpColumn AndAlso Not oRowData Is Nothing) Then
                        oRowData.EmptyRow = True
                    End If
                    iSeq = CInt(strCellData(1))       ' Get the chart RecordID
                    '
                    ' We are loading the current encounter column,
                    ' directly from the container; thus we do not need iSeq
                    '
                    If (UBound(strCellData) > 1) Then   ' Extra data
                        strUnit = "" : strNote = ""
                        For Index = 2 To UBound(strCellData)
                            strField = strCellData(Index)
                            If (strField.Length > 0) Then
                                Select Case strField.Substring(0, 1)
                                    Case "e"
                                        ' If the encounter field is empty, then this is from the server's followup list;
                                        ' Set the description font to FontFollowUp if the view is not flow sheet of a list.
                                        If (Col = myFollowUpColumn) Then
                                            'If (myFlowSheetMode <> FS_LIST) Then
                                            '    'fsFlowSheet.Cell(CellPropertySettings.flexcpFont, NewRow, 0) = mfntFontFollowUp
                                            'End If
                                            ' Set the appearance of the head column (finding caption)
                                            ugRow.Cells(COL_HEAD).Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_FOLLOWUP)
                                        Else
                                            strEtime = strField.Substring(2)    ' Get the encounter time for this finding
                                            ' Get the colData for the column, has etimes
                                            alColData = ugFlowSheet.DisplayLayout.Bands(0).Columns(Col).Tag
                                            If (alColData.Count = 1) Then       ' If only the header is stored,
                                                alColData.Add(strEtime)         ' then add this etime
                                            Else
                                                Dim iElem As Integer            ' Otherwise, if this etime is not in the list,
                                                Dim Found As Boolean = False
                                                For iElem = 1 To alColData.Count - 1
                                                    If (strEtime.Equals(CType(alColData(iElem), String))) Then
                                                        Found = True
                                                        Exit For
                                                    End If
                                                Next
                                                If (Not Found) Then
                                                    alColData.Add(strEtime) ' Add it
                                                End If
                                            End If
                                        End If
                                    Case "n"
                                        If (strField.Length >= 3) Then
                                            strNote = strField.Substring(2)
                                            bOtherData = True
                                        End If
                                    Case "p"
                                        If (strField.Length >= 3) Then
                                            strPrefixCode = strField.Substring(2)
                                        Else
                                            strPrefixCode = ""
                                        End If
                                    Case "q" : strPosneg = Mid$(strField, 3, 1)
                                    Case "v"
                                        If (strField.Length >= 3) Then
                                            strValue = strField.Substring(2)
                                        Else
                                            strValue = ""
                                        End If
                                    Case "u"    ' Unit
                                        If (strField.Length >= 3) Then
                                            strUnit = strField.Substring(2)
                                            bOtherData = True
                                        Else
                                            strUnit = ""
                                        End If
                                    Case "s" : Case "g" ' Ignore sequence  and flags
                                    Case "c"
                                        ' Set other data for chartflag only if <> 0
                                        If (strField.Length >= 3) Then
                                            If (Val(strField.Substring(2)) <> 0) Then
                                                bOtherData = True
                                            End If
                                        End If
                                    Case "L"
                                        If (strField.Length >= 3) Then
                                            RangeScale = CShort(strField.Substring(2))
                                        End If
                                    Case Else
                                        bOtherData = True
                                End Select
                            End If
                        Next
                        '
                        ' If this is not the current encounter column,
                        ' and the column is not the follow up data column,
                        ' put the ExtendedData data into cell info.
                        '
                        If (Col > COL_FINDING AndAlso Col <> myFollowUpColumn) Then
                            oCell = ugRow.Cells(Col).Tag
                            If (Not oCell Is Nothing) Then
                                oCell.CellData.Add(oFinding)
                            Else
                                oCell = New cCellData(oFinding)
                                ugRow.Cells(Col).Tag = oCell
                            End If
                            Dim LineCount As Integer = oCell.CellData.Count
                            If (LineCount > oRowData.LineCount) Then
                                oRowData.LineCount = LineCount
                            End If
                            ' Store the full string in the data space
                            ' since there is more than just a data value;
                            ' we will use this for the cell button.
                            ' CellButton image now handled by ShowButtons=ShowALL
                            ' and setting the combolist property in the BeforeEdit event
                            ' Show the cell button if any finding in the cell has other data
                            If (bOtherData AndAlso myUseCellDataButton) Then
                                oCell.HasCellButton = True
                            End If
                        End If
                    End If
                    If (Not strPrefixCode Is Nothing AndAlso strPrefixCode.Length = 0) Then strPrefixCode = PREFIX_BLANK
                    ' We display either a value, or "P" or "N"
                    ' The server is calculating posneg based on Scale if value
                    If (strPosneg = "+") Then
                        strCell = mstrPositiveText
                        DisplayStyle = CellDisplayStyle.Positive
                    Else
                        strCell = mstrNegativeText
                        DisplayStyle = CellDisplayStyle.Negative
                    End If
                    If (Not strValue Is Nothing AndAlso strValue.Length > 0) Then
                        If (RangeScale = 1 OrElse RangeScale = 7) Then
                            DisplayStyle = CellDisplayStyle.Critical
                        End If
                        strCell = strValue
                        If (Not strUnit Is Nothing AndAlso strUnit.Length > 0) Then
                            oFinding.Display = strCell
                            oFinding.Unit = strUnit     ' Save the unit for easier lookup later
                            strCell = strCell & " " & strUnit
                            ' Code added to track cell units in row same or differing.
                            If (oRowData.Unit.Length = 0) Then
                                oRowData.Unit = strUnit
                                oRowData.UnitsMatch = True
                            ElseIf (Not strUnit.Equals(oRowData.Unit)) Then
                                oRowData.UnitsMatch = False
                            End If
                        End If
                    ElseIf (strNote.Length > 0) Then
                        strCell = strNote
                        oFinding.Display = strCell
                    ElseIf (strPrefixCode <> PREFIX_BLANK) Then
                        strCell = "Some Prefix"     'RetrieveQualifierCaption(0, strPrefixCode)
                        oFinding.Display = strCell
                    Else
                        oFinding.Display = strCell
                    End If
                    If (Not oCell Is Nothing) Then
                        If (oCell.CellData.Count > 1) Then
                            ' This does not work to draw multilines with trimming text.
                            'ugCell.Column.CellAppearance.TextTrimming = TextTrimming.EllipsisWord
                            'ugCell.Column.CellMultiLine = DefaultableBoolean.True
                            strCell = ugCell.Text & vbLf & strCell
                            If (DisplayStyle > oCell.DisplayStyle) Then ' We use the maximum severity styling
                                oCell.DisplayStyle = DisplayStyle
                            Else
                                DisplayStyle = oCell.DisplayStyle
                            End If
                        Else
                            oCell.DisplayStyle = DisplayStyle
                        End If
                    End If
                    ugCell.Value = strCell
                    If (oCell.HasCellButton) Then
                        ugCell.Style = ColumnStyle.EditButton
                        ugCell.Column.ButtonDisplayStyle = UltraWinGrid.ButtonDisplayStyle.Always
                    End If
                    Select Case DisplayStyle
                        Case CellDisplayStyle.Critical
                            ugCell.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_CRITICAL)
                        Case CellDisplayStyle.Negative
                            ugCell.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_NEG)
                        Case CellDisplayStyle.Positive
                            ugCell.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_POS)
                    End Select
            End Select
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub


    Private Sub ParseHdrInfo(ByVal NodeKey As String)
        Try
            Dim bolHdrChange As Boolean = False
            Dim hdrInfo() As String = NodeKey.Split(",")
            If (hdrInfo(0) <> mstrDateFm) Then
                mstrDateFm = hdrInfo(0)
                bolHdrChange = True
            End If
            If (hdrInfo(1) <> mstrDateTo) Then
                mstrDateTo = hdrInfo(1)
                bolHdrChange = True
            End If
            If (hdrInfo(2) <> mstrTimeFm) Then
                mstrTimeFm = hdrInfo(2)
                bolHdrChange = True
            End If
            If (hdrInfo(3) <> mstrTimeTo) Then
                mstrTimeTo = hdrInfo(3)
                bolHdrChange = True
            End If
            'If (CType(Asc(hdrInfo(4)), DatePrecisions) <> myColumnPrecision) Then
            '    myColumnPrecision = CType(Asc(hdrInfo(4)), DatePrecisions)
            '    'SetColumnPrecisionMenu(myColumnPrecision)
            '    bolHdrChange = True
            'End If
            'If (CType(Val(hdrInfo(5)), SortOrder) <> mySortOrder) Then
            '    mySortOrder = CType(Val(hdrInfo(5)), SortOrder)
            '    'SetColumnOrderMenu(mySortOrder)
            '    bolHdrChange = True
            'End If
            'If (hdrInfo.Length > 6) Then    ' This parameter added for 3Q2007
            '    Dim Mode As DataFilterMode = CType(hdrInfo(6), DataFilterMode)
            '    'SetFilterModeMenu(Mode)
            'End If
            'If (bolHdrChange) Then SetStatusInfo()
        Catch ex As Exception

        End Try
    End Sub

    ''' <summary>
    ''' Procedure sets the Appearance object for each row after the grid is built with data
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ColorRows()
        ' Row Types indicates which types of rows are enumerated: Data rows, GroupBy rows, or both.
        Dim rowTypes As GridRowType = Infragistics.Win.UltraWinGrid.GridRowType.DataRow Or Infragistics.Win.UltraWinGrid.GridRowType.GroupByRow
        ' Get a Row Enumerator that will return all rows in the entire grid.
        ' This includes all rows in the root band as well as all child bands down
        ' to the lowest level.
        Dim enumerator As IEnumerable = ugFlowSheet.Rows.GetRowEnumerator(rowTypes, Nothing, Nothing)
        Dim row As UltraGridRow
        For Each row In enumerator
            ' Since this enumerator can return Data rows or GroupBy rows,
            ' check IsGroupByRow to determine which one we are dealing with.
            If (row.IsGroupByRow) Then
                If (row.ParentRow Is Nothing) Then
                    ' This must be a group row
                    row.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_GROUP)
                Else
                    ' This must be a subgroup row (system row)
                    row.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_SYSTEM)
                End If
            Else
                ' This is a data row
                row.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_HEAD)
                Dim oRowData As cFlowRowData = row.Tag
                If (oRowData.LineCount > 1) Then
                    row.Height *= oRowData.LineCount
                End If
            End If
        Next
        'With ugFlowSheet.DisplayLayout
        '    .RowConnectorStyle = RowConnectorStyle.Solid
        '    .RowConnectorColor = Color.Red
        '    .Bands.Layout.RowConnectorStyle = RowConnectorStyle.Solid
        'End With
        'Dim band As UltraGridBand = ugFlowSheet.DisplayLayout.Bands(0)
        'With band
        '    ' You can hide the column headers by setting ColHeadersVisible to false. By
        '    ' default, column headers are visible.
        '    '.ColHeadersVisible = False

        '    ' You can hide the group headers by setting GroupHeadersVisible to false. By
        '    ' default, group headers are visible.
        '    'band.GroupHeadersVisible = False

        '    ' You can also show the band header which by default is hidden by setting
        '    ' the HeaderVisible property to true.
        '    'band.HeaderVisible = True

        '    ' You can also set the caption of the band header.
        '    'band.Header.Caption = "Customers Table"
        '    ' You can use the HeaderPlacement property to specify how headers are displayed.
        '    'band.Override.HeaderPlacement = HeaderPlacement.FixedOnTop

        '    ' You can also set the apperance of the band header using Appearance property
        '    ' off the header.
        '    'band.Header.Appearance.BackColor = myHeaderRowBackColor
        '    'band.Header.Appearance.ForeColor = Color.LightYellow
        '    '.Header.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_HEADER)

        '    '.Override.FixedHeaderAppearance.BackColor = myHeaderRowBackColor
        '    '.Override.FixedHeaderAppearance.BackColor2 = myHeaderRowBackColor
        '    '.Override.HeaderAppearance.BackColor = myHeaderRowBackColor
        '    '.Override.HeaderAppearance.BackColor2 = myHeaderRowBackColor
        'End With
    End Sub

    ''' <summary>
    ''' Procedure used to delete a current finding from a data row
    ''' </summary>
    ''' <param name="Row">The UltraGridRow object defining the data row.</param>
    ''' <remarks></remarks>
    Private Sub DeleteCurrentFindingFromFlowSheet(ByVal Row As UltraGridRow)
        Dim Flag As Short
        Dim ugCell As UltraGridCell = Row.Cells(COL_FINDING)
        With ugFlowSheet
            Flag = CType(Row.Tag, cFlowRowData).Flag
            ' Set it to empty so our tests on empty do not fail
            ugCell.Tag = Nothing
            ugCell.Value = String.Empty
        End With
        AddIconsToPosNeg(Row, FindingButtonState.fscButtonNone, False)
    End Sub

    Private Sub EmitRowInfo()
        ' Row Types indicates which types of rows are enumerated: Data rows, GroupBy rows, or both.
        ' In this case, return both.
        Dim rowTypes As GridRowType = Infragistics.Win.UltraWinGrid.GridRowType.DataRow Or Infragistics.Win.UltraWinGrid.GridRowType.GroupByRow

        ' Get a Row Enumerator that will return all rows in the entire grid.
        ' This includes all rows in the root band as well as all child bands down
        ' to the lowest level.
        Dim enumerator As IEnumerable = ugFlowSheet.Rows.GetRowEnumerator(rowTypes, Nothing, Nothing)
        Dim row As UltraGridRow

        'Debug.WriteLine("===== Start of row information:")
        For Each row In enumerator
            ' Since this enumerator can return Data rows or GroupBy rows,
            ' check IsGroupByRow to determine which one we are dealing with.
            If (row.IsGroupByRow) Then
                If (row.ParentRow Is Nothing) Then
                    '' This must be a group row
                    'Debug.Write("Group: " & row.Description)
                    'Debug.WriteLine(" <" & row.ChildBands(0).Rows(0).ChildBands(0).Rows(0).Cells(0).Tag & ">")
                Else
                    '' This must be a subgroup row
                    'Debug.Write("SubGroup: " & row.Description & " of " & row.ParentRow.Description)
                    'Debug.WriteLine(" <" & row.ChildBands(0).Rows(0).Cells(1).Tag & ">")
                End If
            Else
                ' This is a data row
                'Debug.WriteLine(row.Cells(0).Value)
            End If
        Next
    End Sub

    Private Sub FinishGridLayout(ByVal ugGrid As UltraGrid)
        If (ugGrid.DisplayLayout.Bands.Count > 0) Then
            Dim Band As UltraGridBand = ugGrid.DisplayLayout.Bands(0)
            If (Band.Columns.Count > COL_FINDING) Then
                With Band
                    Dim Index As Integer
                    .Header.Appearance.ThemedElementAlpha = Alpha.Transparent
                    'These next two eliminate the groupby connection strip, but can't indent the data rows.
                    '.IndentationGroupByRow = 0
                    '.IndentationGroupByRowExpansionIndicator = 10
                    .Override.FixedHeaderIndicator = FixedHeaderIndicator.None      ' Don't show the user any pushpins
                    .Override.HeaderClickAction = HeaderClickAction.ExternalSortSingle  ' Inhibit sorting

                    ' Assign an instance of IComparer to SortComparer property.
                    Dim ugCol As UltraGridColumn
                    Dim Style As Infragistics.Win.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_HEADER)
                    For Index = COL_GROUP To COL_FINDING
                        ugCol = .Columns(Index)
                        ugCol.Header.Fixed = True                         ' These are all fixed columns
                        ugCol.AutoSizeMode = ColumnAutoSizeMode.None
                        If (Index < COL_FINDING) Then
                            ugCol.Header.Caption = ""
                        End If
                        If (Index <= COL_SUBGROUP) Then
                            .SortedColumns.Add(.Columns(Index), False, True)    ' Can only group columns if they are added to the SortedColumns collection
                            ugCol.GroupByMode = GroupByMode.Text          ' Specify grouping by the text
                            ' Apparently we do not need to do this as a workaround if we set e.Cancel=True in BeforeSortChange
                            'ugcol.SortComparer = New MySortComparer()     ' Set a comparer to suppress actual sorting, as we have the grid already ordered
                        End If
                    Next
                    '.Override.CellClickAction = CellClickAction.CellSelect  ' We do not go into edit mode
                    For Each ugCol In .Columns
                        ugCol.Header.Appearance = Style
                    Next
                End With
            End If
        End If
        ResizeGridColumns()
        ColorRows()
        ugGrid.Rows.ExpandAll(True)     ' This has all groups expanded
        Debug.WriteLine("Magenta=" & System.Drawing.ColorTranslator.ToOle(Color.Magenta).ToString)
        'EmitRowInfo()
    End Sub

    ''' <summary>
    ''' Here simply to check the formating
    ''' </summary>
    ''' <param name="strTime"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function FormatHeaderTime(ByVal strTime As String) As String
        'If (strTime.Length > 8) Then
        '    Return MedClient.FormatDate(strTime, mstrDayFormat)
        'ElseIf (strTime.Length > 0) Then
        '    Return MedClient.FormatDate(strTime, "")
        'Else
        '    Return ""
        'End If
        Return strTime
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This method is used to create our custom styles for the flow sheet cells
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Private Sub InitializeFlowSheetStyles()
        Dim Style As Infragistics.Win.Appearance
        Try
            With ugFlowSheet.DisplayLayout
                If (.Appearances.Count = 0) Then
                    Style = .Appearances.Add(STYLE_HEADER)
                    Style.BackColor = myHeaderRowBackColor
                    Style.BackColor2 = myHeaderRowBackColor
                    Style.ForeColor = myHeaderRowForeColor
                    'myStyle.Border.Color = System.Drawing.Color.DarkGray
                    'myStyle.Border.Width = 1
                    'myStyle.Border.Style = BorderStyleEnum.Flat

                    Style = .Appearances.Add(STYLE_BORDER)
                    'myStyle.Border.Color = System.Drawing.Color.LightGray
                    'myStyle.Border.Width = 1
                    'myStyle.Border.Style = BorderStyleEnum.Flat
                    'myStyle.BackColor = myFindingRowBackColor
                    'myStyle.ForeColor = myFindingDescriptionForeColor

                    Style = .Appearances.Add(STYLE_HEAD)
                    Style.BackColor = myFindingRowBackColor
                    Style.BackColor2 = myFindingRowBackColor
                    Style.ForeColor = myFindingDescriptionForeColor

                    Style = .Appearances.Add(STYLE_NEG)
                    Style.BackColor = myFindingRowBackColor
                    Style.BackColor2 = myFindingRowBackColor
                    Style.ForeColor = myNegativeTextForeColor
                    'myStyle.Border.Color = System.Drawing.Color.LightGray
                    'myStyle.Border.Width = 1
                    'myStyle.Border.Style = BorderStyleEnum.Flat
                    'myStyle.ImageAlign = ImageAlignEnum.RightCenter

                    Style = .Appearances.Add(STYLE_POS)
                    Style.BackColor = myFindingRowBackColor
                    Style.ForeColor = myPositiveTextForeColor
                    'myStyle.Border.Color = System.Drawing.Color.LightGray
                    'myStyle.Border.Width = 1
                    'myStyle.Border.Style = BorderStyleEnum.Flat
                    'myStyle.ImageAlign = ImageAlignEnum.RightCenter

                    Style = .Appearances.Add(STYLE_GROUP)
                    Style.BackColor = myGroupRowBackColor
                    Style.BackColor2 = myGroupRowBackColor
                    Style.ForeColor = myGroupRowForeColor

                    Style = .Appearances.Add(STYLE_SYSTEM)
                    Style.BackColor = mySystemRowBackColor
                    Style.BackColor2 = mySystemRowBackColor
                    Style.ForeColor = mySystemRowForeColor

                    Style = .Appearances.Add(STYLE_CRITICAL)
                    Style.BackColor = myCriticalTextBackColor
                    Style.BackColor2 = myCriticalTextBackColor
                    Style.ForeColor = myCriticalTextForeColor
                    'myStyle.Border.Color = System.Drawing.Color.LightGray
                    'myStyle.Border.Width = 1
                    'myStyle.Border.Style = BorderStyleEnum.Flat
                    'myStyle.ImageAlign = ImageAlignEnum.RightCenter

                    Style = .Appearances.Add(STYLE_FOLLOWUP)
                    Style.ForeColor = myFollowUpDescriptionForeColor
                End If
            End With
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    ''' <summary>
    ''' Procedure to initialize the Infragistics UltraWinGrid
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub InitializeGrid()
        ugFlowSheet.UseOsThemes = DefaultableBoolean.False ' Allows us to control the column header colors
        With ugFlowSheet.DisplayLayout
            .Reset()
            .BorderStyle = UIElementBorderStyle.None
            .CaptionVisible = DefaultableBoolean.False
            .UseFixedHeaders = True                             ' This allows setting up columns fixed from scrolling horizontally
            .ViewStyle = ViewStyle.SingleBand
            .ViewStyleBand = ViewStyleBand.OutlookGroupBy       ' This sets up for Outlook style grouping needed for group and subgroup
            .GroupByBox.Hidden = True       ' This hides the GroupBy box from the user
            .MaxColScrollRegions = 1        ' This keeps the user from splitting the grid horizontally
            .MaxRowScrollRegions = 1        ' This keeps the user from splitting the grid vertically
            .Override.RowSelectors = DefaultableBoolean.False   ' Set to Default to add a row selector
            .Override.RowSizing = RowSizing.Fixed               ' We will perform row sizing
            .Override.CellAppearance.TextTrimming = TextTrimming.EllipsisCharacter
            .Override.CellClickAction = CellClickAction.CellSelect  ' We do not go into edit mode
            .Override.GroupByRowDescriptionMask = "[value]"     ' Use only the text on groupby rows
            .Override.HeaderStyle = HeaderStyle.Default
            '.Override.BorderStyleHeader = UIElementBorderStyle.None
            .MaxColScrollRegions = 1
            .MaxRowScrollRegions = 1

            .Override.SelectTypeCell = SelectType.None
            .Override.SelectTypeRow = SelectType.Extended
            .Override.SelectTypeGroupByRow = SelectType.Extended    ' Keeps GroupBy rows (group, subgroup) from selecting

            .Override.TipStyleCell = TipStyle.Hide              ' Turn off the grid's cell tips
            .Override.TipStyleHeader = TipStyle.Hide            ' Turn off header tips
            .ScrollBounds = ScrollBounds.ScrollToFill
            .ScrollStyle = ScrollStyle.Immediate
            ' These seem to have no effect
            .RowConnectorStyle = RowConnectorStyle.Solid
            .RowConnectorColor = Color.Red
            '.Bands.Layout.RowConnectorStyle = RowConnectorStyle.Solid
            .EmptyRowSettings.ShowEmptyRows = False
        End With
        ' Attach my DrawFilter for custom drawing the cell text
        ' This is needed to handle the multi-line cells that contain more than one finding
        ugFlowSheet.DrawFilter = myDrawFilter

        ugFlowSheet.UpdateMode = UpdateMode.OnUpdate
        InitializeFlowSheetStyles()
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This procedure loads a cell of the current encounter column with a finding.
    ''' The finding can come from the encounter load, or from an add/edit using a pos-neg button.
    ''' </summary>
    ''' <param name="Row">This is the grid row of the finding.</param>
    ''' <param name="objFinding">This contains the cFlowFinding object that is added/edited.</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	10/18/2006	Changed to monitor units
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Private Sub LoadCurrentCell(ByVal Row As UltraGridRow, ByVal objFinding As cFlowFinding)
        Dim strValue As String = ""
        Dim strData As String
        Dim strNote As String = ""
        Dim strPrefixCode As String = ""
        Dim strUnit As String = ""
        Dim Style As String = ""
        Dim Flag, RangeScale As Short
        Dim bolReverse As Boolean
        Dim DisplayStyle As CellDisplayStyle
        Dim oRowData As cFlowRowData
        'Dim CellColor As System.Drawing.Color

        With ugFlowSheet
            oRowData = Row.Tag
            If (objFinding Is Nothing) Then
                objFinding = Row.Cells(COL_FINDING).Tag
            Else
                Row.Cells(COL_FINDING).Tag = objFinding
            End If
            Flag = oRowData.Flag  ' Get property flag from objFinding
            bolReverse = False
            ' Need to set the finding cell text
            With objFinding
                If (Not .Chart Is Nothing) Then
                    With .Chart
                        strValue = .Value
                        strUnit = .Unit
                        strNote = .Note
                        strPrefixCode = .Prefix
                        RangeScale = .RangeScale
                    End With
                    If (RangeScale = 0) Then
                        ' Use result to set color
                        If (.Chart.Result = RESULT_NEG) Then
                            .ButtonState = FindingButtonState.fscButtonNegative     ' 2nd button
                        Else
                            .ButtonState = FindingButtonState.fscButtonPositive     ' 1st button
                        End If
                    Else
                        ' Scale of 4 indicates a normal value
                        .ButtonState = IIf(RangeScale = 4, FindingButtonState.fscButtonNegative, FindingButtonState.fscButtonPositive)
                    End If
                End If
                If (.ButtonState = FindingButtonState.fscButtonPositive) Then   ' If 1st button
                    If (bolReverse) Then
                        strData = mstrNegativeText
                        'CellColor = myNegativeTextForeColor
                        DisplayStyle = CellDisplayStyle.Negative
                    Else
                        strData = mstrPositiveText
                        'CellColor = myPositiveTextForeColor
                        DisplayStyle = CellDisplayStyle.Positive
                    End If
                Else        ' 2nd button
                    If (bolReverse) Then
                        strData = mstrPositiveText
                        'CellColor = myPositiveTextForeColor
                        DisplayStyle = CellDisplayStyle.Positive
                    Else
                        strData = mstrNegativeText
                        'CellColor = myNegativeTextForeColor
                        DisplayStyle = CellDisplayStyle.Negative
                    End If
                End If
            End With
            If (Not strValue Is Nothing AndAlso strValue.Length > 0) Then
                If (RangeScale = 1 OrElse RangeScale = 7) Then
                    DisplayStyle = CellDisplayStyle.Critical
                End If
                strData = strValue
                If (Not strUnit Is Nothing AndAlso strUnit.Length > 0) Then
                    ' Code added to track cell units in row same or differing.
                    If (oRowData.Unit.Length = 0) Then
                        oRowData.Unit = strUnit
                        oRowData.UnitsMatch = True
                    ElseIf (Not strUnit.Equals(oRowData.Unit)) Then
                        oRowData.UnitsMatch = False
                    End If
                    ' Add the units only if they are not supposed to be in the header
                    If (Not (mbolUnitsInHeader And oRowData.UnitsMatch)) Then
                        strData = strData & " " & strUnit
                    End If
                End If
            ElseIf (strNote.Length > 0) Then
                strData = strNote.Substring(0, Math.Min(32, strNote.Length))
            ElseIf (strPrefixCode <> PREFIX_BLANK) Then
                strData = RetrieveQualifierCaption(0, strPrefixCode)
            End If
            Dim ugCell As UltraGridCell = Row.Cells(COL_FINDING)
            ugCell.Value = strData
            Select Case DisplayStyle
                Case CellDisplayStyle.Critical
                    ugCell.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_CRITICAL)
                Case CellDisplayStyle.Negative
                    ugCell.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_NEG)
                Case CellDisplayStyle.Negative : Style = STYLE_POS
                    ugCell.Appearance = ugFlowSheet.DisplayLayout.Appearances(STYLE_POS)
            End Select
            AddIconsToPosNeg(Row, objFinding.ButtonState, bolReverse)
        End With
    End Sub

    Private Sub ProcessBuildListDirect(ByVal rList As ArrayList)
        ' Procedure to receive recordsets from the Medcin control
        Dim Description As String
        Dim oItem As ResponseItem
        Try
            mbolLoading = True
            ugFlowSheet.DataSource = Nothing
#If DEBUGBUILDLIST Then
            FileOpen(1, "Grid.txt", OpenMode.Output, OpenAccess.Write, OpenShare.Shared)
#End If
            ugFlowSheet.BeginUpdate()
            For Each oItem In rList
                With oItem
                    Description = .Description
#If DEBUGBUILDLIST Then
                    WriteLine(1, .Medcinid, .Nodekey, Description, .Prefix, .Subs, .TermType, .Flags, .Dx)
#End If
                    ' Findings with "**" in the text are Copyright findings
                    ' which do not have to be presented for user selection
                    If (InStr(Description, "**") = 0) Then
                        AddRecordToFlowSheet(.Medcinid, .Nodekey, Description, .Prefix, .Subs, .TermType, .Flags, .Dx)
                    End If
                End With
            Next
            FinishGridLayout(ugFlowSheet)
            'With ugFlowSheet
            'For Each Row In .Rows
            '    If (Row.IsGroupByRow) Then
            '        Dim cBand As UltraGridChildBand
            '        For Each cBand In Row.ChildBands
            '            For Each cRow In cBand.Rows
            '                If (cRow.IsGroupByRow) Then
            '                    Dim dBand As UltraGridChildBand
            '                    For Each dBand In cRow.ChildBands
            '                        For Each dRow In dBand.Rows
            '                            'AddIconsToPosNeg(Row, FindingButtonState.fscButtonNone, (mbolSwapNeg And ((Flags And PROP_NORM) <> 0)))
            '                            AddIconsToPosNeg(dRow, FindingButtonState.fscButtonNone, False)
            '                        Next
            '                    Next
            '                End If
            '            Next
            '        Next
            '    End If
            'Next
            '    ' This will loop through all the rows of the grid, including rows from child bands, getting just data rows.
            '    For Each Row In .Rows.GetRowEnumerator(Infragistics.Win.UltraWinGrid.GridRowType.DataRow, Nothing, Nothing)
            '    Next Row
            'End With
#If DEBUGBUILDLIST Then
            FileClose(1)
#End If
        Catch ex As Exception
            PostError(ex)
        Finally
            ugFlowSheet.EndUpdate()
            mbolLoading = False
        End Try
    End Sub

    ''' <summary>
    ''' Handle user action of clicking a pos or neg finding button
    ''' </summary>
    ''' <param name="Row">Row is the UltraGridRow object of the row clicked</param>
    ''' <param name="Col">Col is COL_POS or COL_NEG</param>
    ''' <remarks></remarks>
    Private Sub ProcessButtonAction(ByVal Row As UltraGridRow, ByVal Col As Integer)
        Dim objFinding As cFlowFinding
        Dim objChart As MedChart
        Dim oRowData As cFlowRowData

        Try
            With ugFlowSheet
                Dim fbsState As FindingButtonState
                Dim Flag As Short
                ' Medcinid stored in rowData CFlowRowData object
                oRowData = Row.Tag
                myCurrentMedcinid = oRowData.Medcinid
                ' Current button state
                fbsState = Row.Cells(COL_POS).Tag

                If (fbsState = FindingButtonState.fscButtonNone) Then
                    ' This is an ADD
                    objFinding = New cFlowFinding
                    objFinding.Medcinid = myCurrentMedcinid
                    Flag = oRowData.Flag
                    If (Col = COL_POS) Then
                        fbsState = FindingButtonState.fscButtonPositive
                    Else
                        fbsState = FindingButtonState.fscButtonNegative
                    End If
                    objFinding.ButtonState = fbsState
                    objFinding.Group = oRowData.Group
                    ' Get a new chart record
                    objChart = New MedChart
                    With objChart
                        .Medcinid = myCurrentMedcinid
                        .Prefix = oRowData.Prefix
                        'If (oRowData.Group = MedcinGroups.ReviewOfSystems) Then .Flag = ChartFlags.ROS
                        '' Set the result based on positive|negative
                        'If (fbsState = FindingButtonState.fscButtonPositive) Then
                        '    .Result = RESULT_BLANK
                        'Else
                        '    If ((Flag And PROP_NORM) <> 0) Then
                        '        .Result = RESULT_ABN
                        '    Else
                        '        .Result = RESULT_NEG
                        '    End If
                        'End If
                    End With
                    objFinding.Chart = objChart
                    RaiseFindingAction(FindingActionMode.fscAddFinding, objFinding)
                    ' Container must set the client key to acknowledge add
                    If (objFinding.ClientKey.Length > 0) Then
                        LoadCurrentCell(Row, objFinding)
                    End If

                ElseIf ((fbsState = FindingButtonState.fscButtonPositive And Col = COL_POS) _
                  Or (fbsState = FindingButtonState.fscButtonNegative And Col = COL_NEG)) Then
                    ' This is a DELETE
                    objFinding = Row.Cells(COL_FINDING).Tag
                    RaiseFindingAction(FindingActionMode.fscDeleteFinding, objFinding)
                    ' Container must clear the client key to acknowledge delete
                    If (objFinding.ClientKey.Length = 0) Then
                        DeleteCurrentFindingFromFlowSheet(Row)
                    End If

                Else
                    ' This is an EDIT
                    objFinding = Row.Cells(COL_FINDING).Tag
                    If (Col = COL_POS) Then
                        fbsState = FindingButtonState.fscButtonPositive
                    Else
                        fbsState = FindingButtonState.fscButtonNegative
                    End If
                    objFinding.ButtonState = fbsState
                    RaiseFindingAction(FindingActionMode.fscEditFinding, objFinding)
                    LoadCurrentCell(Row, objFinding)
                End If
            End With

        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    ''' <summary>
    ''' Deal with the user clicking a column header
    ''' </summary>
    ''' <param name="Col">The index of the column clicked.</param>
    ''' <remarks></remarks>
    Private Sub ProcessHeaderClick(ByVal Col As Integer)
    End Sub


    ''' <summary>
    ''' Method called by AfterBuildFlowSheet and resize of flowsheet
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ResizeGridColumns()
        Dim Col, Width, wid, widcol As Integer
        Try
            With ugFlowSheet
                Width = .Width '- 40
                widcol = Width \ 10        ' All columns to 10%
                Dim Band As UltraGridBand = .DisplayLayout.Bands(0)
                If (Band.Columns.Count > COL_FINDING) Then
                    With Band
                        For Col = COL_HEAD To .Columns.Count - 1
                            Select Case Col
                                Case COL_HEAD : wid = Width \ 4 ' header column to 25%
                                Case COL_POS, COL_NEG : wid = 20 ' Button cols for pic
                                Case Else
                                    wid = widcol        ' All columns to 10%
                            End Select
                            With .Columns(Col)
                                .Width = wid
                                .Layout.Appearance.TextHAlign = Infragistics.Win.HAlign.Left ' All cols left aligned
                            End With
                        Next
                    End With
                End If
            End With
        Catch ex As Exception

        End Try
    End Sub

#Region "Private Classes"
    ''' -----------------------------------------------------------------------------
    ''' Project	 : FlowSheetControl
    ''' Class	 : Controls.FlowSheet.cCellData
    ''' 
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Class used to store finding data for a history cell
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	6/5/2006	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Private Class cCellData
        Private myHasCellButton As Boolean
        Private myCellStyle As CellDisplayStyle
        Private myCellData As ArrayList
        Sub New(ByVal finding As cCellFinding)
            myCellData = New ArrayList
            myCellData.Add(finding)
        End Sub
        Function CellData() As ArrayList
            Return myCellData
        End Function
        Property HasCellButton() As Boolean
            Get
                Return myHasCellButton
            End Get
            Set(ByVal Value As Boolean)
                myHasCellButton = Value
            End Set
        End Property
        Property DisplayStyle() As CellDisplayStyle
            Get
                Return myCellStyle
            End Get
            Set(ByVal Value As CellDisplayStyle)
                myCellStyle = Value
            End Set
        End Property
    End Class

    ''' -----------------------------------------------------------------------------
    ''' Project	 : FlowSheetControl
    ''' Class	 : Controls.FlowSheet.cCellFinding
    ''' 
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This class is used to hold data for one history pool finding added to a cell
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	10/18/2006	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Friend Class cCellFinding
        Private myExtendedData As String
        Private myDisplay As String
        Private myUnit As String
        Sub New(ByVal data As String)
            myExtendedData = data
        End Sub
        Property Display() As String
            Get
                Return myDisplay
            End Get
            Set(ByVal Value As String)
                myDisplay = Value
            End Set
        End Property
        Property Unit() As String
            Get
                Return myUnit
            End Get
            Set(ByVal Value As String)
                myUnit = Value
            End Set
        End Property
        ReadOnly Property ExtendedData() As String
            Get
                Return myExtendedData
            End Get
        End Property
    End Class

    Private Class cFlowRowData
        ' Class used in the flow sheet to hold row data
        Private myMedcinid As Integer
        Private myLineCount As Integer
        Private myDescription As String
        Private myNodekey As String
        Private myPrefix As String
        Private myUnit As String = ""
        Private myGroup As Short    'MedcinGroups
        Private mySubGroup As Short
        Private myFlag As Short     ' Flag property of Medcinid
        Private myHiddenRow As Boolean
        Private myEmptyRow As Boolean
        Private myUnitsMatch As Boolean
        Sub New()

        End Sub
        Sub New(ByVal medcinid As Integer)
            myMedcinid = medcinid
        End Sub
        Sub New(ByVal medcinid As Integer, ByVal nodekey As String)
            myMedcinid = medcinid
            myNodekey = nodekey
        End Sub
        Sub New(ByVal medcinid As Integer, ByVal nodekey As String, ByVal group As Short)
            myMedcinid = medcinid
            myNodekey = nodekey
            myGroup = group
        End Sub
        Sub New(ByVal medcinid As Integer, ByVal nodekey As String, ByVal desc As String, ByVal group As Short, ByVal subgroup As Short)
            myMedcinid = medcinid
            myNodekey = nodekey
            myDescription = desc
            myGroup = group
            mySubGroup = subgroup
        End Sub
        Sub New(ByVal group As Short, ByVal subgroup As Short)
            myGroup = group
            mySubGroup = subgroup
        End Sub
        Sub New(ByVal group As Short)
            myGroup = group
        End Sub
        Property Medcinid() As Integer
            Get
                Return myMedcinid
            End Get
            Set(ByVal Value As Integer)
                myMedcinid = Value
            End Set
        End Property
        Property LineCount() As Integer
            Get
                Return myLineCount
            End Get
            Set(ByVal value As Integer)
                myLineCount = value
            End Set
        End Property
        Property Description() As String
            Get
                Return myDescription
            End Get
            Set(ByVal Value As String)
                myDescription = Value
            End Set
        End Property
        Property Nodekey() As String
            Get
                Return myNodekey
            End Get
            Set(ByVal Value As String)
                myNodekey = Value
            End Set
        End Property
        Property Prefix() As String
            Get
                Return myPrefix
            End Get
            Set(ByVal Value As String)
                myPrefix = Value
            End Set
        End Property
        Property Unit() As String
            Get
                Return myUnit
            End Get
            Set(ByVal Value As String)
                myUnit = Value
            End Set
        End Property
        Property Group() As Short
            Get
                Return myGroup
            End Get
            Set(ByVal Value As Short)
                myGroup = Value
            End Set
        End Property
        Property SubGroup() As Short
            Get
                Return mySubGroup
            End Get
            Set(ByVal Value As Short)
                mySubGroup = Value
            End Set
        End Property
        Property Flag() As Short
            Get
                Return myFlag
            End Get
            Set(ByVal Value As Short)
                myFlag = Value
            End Set
        End Property
        Property HiddenRow() As Boolean
            Get
                Return myHiddenRow
            End Get
            Set(ByVal Value As Boolean)
                myHiddenRow = Value
            End Set
        End Property
        Property EmptyRow() As Boolean
            Get
                Return myEmptyRow
            End Get
            Set(ByVal Value As Boolean)
                myEmptyRow = True
            End Set
        End Property
        Property UnitsMatch() As Boolean
            Get
                Return myUnitsMatch
            End Get
            Set(ByVal Value As Boolean)
                myUnitsMatch = Value
            End Set
        End Property
    End Class
#End Region


    Private Sub ugFlowSheet_BeforeCellActivate(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.CancelableCellEventArgs) Handles ugFlowSheet.BeforeCellActivate
        ''Debug.WriteLine("BeforeCellActivate")
    End Sub

    Private Sub ugFlowSheet_BeforeEnterEditMode(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles ugFlowSheet.BeforeEnterEditMode
        e.Cancel = True ' Need this to keep cell from entering edit mode on cell button click
    End Sub

    Private Sub ugFlowSheet_BeforePrint(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.CancelablePrintEventArgs) Handles ugFlowSheet.BeforePrint
        Debug.WriteLine("ugFlowSheet_BeforePrint")
    End Sub

    Private Sub ugFlowSheet_BeforeRowActivate(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.RowEventArgs) Handles ugFlowSheet.BeforeRowActivate
        ''Debug.WriteLine("BeforeRowActivate")
    End Sub

    Private Sub ugFlowSheet_BeforeSelectChange(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.BeforeSelectChangeEventArgs) Handles ugFlowSheet.BeforeSelectChange
        'Debug.WriteLine("BeforeSelectChange")
    End Sub


    Private Sub ugFlowSheet_BeforeSortChange(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.BeforeSortChangeEventArgs) Handles ugFlowSheet.BeforeSortChange
        ' If loading the grid, allow the sorting used for GroupBy
        If (mbolLoading) Then Exit Sub
        ' Do not allow any user column sorting
        e.Cancel = True
    End Sub

    Private Sub ugFlowSheet_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ugFlowSheet.Click
        ' Use this procedure to detect the user clicking on a column header
        ' Retrieve the UIElement from the location of the MouseUp
        Dim ugUIElement As UIElement = ugFlowSheet.DisplayLayout.UIElement.ElementFromPoint(myMouse.Location)
        ' Get the row that contains this element.
        Dim ugRow As UltraGridRow = ugUIElement.GetContext(GetType(UltraGridRow))
        ' A column header will not be grid row type
        If (Not ugRow Is Nothing) Then
            If ugRow.IsDataRow Then Exit Sub
            tssStatus.Text = "Clicked GroupBy Row: Button: " & myMouse.Button.ToString
            Exit Sub
        End If
        ' Retrieve the Column from the UIElement
        Dim ugColumn As UltraGridColumn = ugUIElement.GetContext(GetType(UltraGridColumn))
        ' If not a column, then exit
        If (ugColumn Is Nothing) Then Exit Sub
        Dim Col As Integer = ugColumn.Index

        ' This reflects the column index and mouse button
        tssStatus.Text = "Clicked Header Column: " & ugColumn.Index & " Button: " & myMouse.Button.ToString

        Try

            myEventRow = Nothing                    ' None until we verify
            'If (mbolIgnoreClick) Then
            '    mbolIgnoreClick = False
            '    Exit Sub
            'End If
            If (myMouse.Button = Windows.Forms.MouseButtons.Right) Then
                '' Show the row context menu if col=0 and allowed
                If (Col = 0 AndAlso mbolAllowRowMenu) Then
                    '    ShowRowContextMenu(Row, New cFlowFinding)
                    '    ' Otherwise Show the context menu if application is allowing it
                ElseIf (myHeaderActions And ColumnHeaderActions.AllowHeaderMenu) Then
                    '    ' Validate any menu items as needed
                    '    HeaderMenu.MenuItems(HMENU_EMPTYROWS).Enabled = (myFollowUpColumn > 0)
                    '    HeaderMenu.Show(fsFlowSheet, omdMouse.pt)
                End If
            Else
                ProcessHeaderClick(Col)
            End If
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    ''' <summary>
    ''' Event raised when the user clicks on a data row cell (COL_HEAD to last column)
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks>ClickCell events are not raised if the user clicks a GroupBy row</remarks>
    Private Sub ugFlowSheet_ClickCell(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.ClickCellEventArgs) Handles ugFlowSheet.ClickCell
        Dim strInfo As String = String.Empty
        strInfo = "Cell Row = " & e.Cell.Row.ListIndex
        strInfo = strInfo & "; Cell Col = " & e.Cell.Column.Index
        strInfo = strInfo & "; Has Data = " & Not e.Cell.Tag Is Nothing
        Dim oTag As Object = e.Cell.Row.Tag
        If (Not oTag Is Nothing) Then
            strInfo = strInfo & "; Medcinid = " & DirectCast(oTag, cFlowRowData).Medcinid
        End If
        strInfo = strInfo & "; Button Clicked = " & myMouse.Button.ToString
        tssStatus.Text = strInfo

        myEventRow = Nothing                    ' None until we verify
        Dim Col As Integer = e.Cell.Column.Index
        Dim objFinding As cFlowFinding
        Dim objRowData As cFlowRowData
        Dim ugRow As UltraGridRow = e.Cell.Row
        Try
            If (Col = COL_HEAD) Then            ' This is the header column
                '
                ' Request to set the Medcinid of this row as current
                ' Advise container for possibly graphing the row data
                '
                objFinding = New cFlowFinding
                With ugFlowSheet
                    'If (.Rows(Row).IsNode) Then
                    '    RowLevel = .Rows(Row).Node.Level
                    'Else
                    '    RowLevel = LVL_FINDING
                    'End If
                    'If (RowLevel = LVL_FINDING) Then
                    ' Medcinid stored in rowData CFlowRowData object
                    objRowData = ugRow.Tag
                    With objRowData
                        myCurrentMedcinid = .Medcinid
                        objFinding.Medcinid = myCurrentMedcinid
                        objFinding.Group = .Group
                        objFinding.SubGroup = .SubGroup
                    End With
                    myEventRow = ugRow
                    'Else
                    '' Group and subgroup data stored in a cFlowSubGroupRow object
                    'objRowData = .Rows(Row).UserData
                    'With objRowData
                    '    objFinding.Group = .Group
                    '    If (RowLevel = LVL_BODYSYS) Then
                    '        objFinding.SubGroup = .SubGroup
                    '    End If
                    'End With
                    'myCurrentMedcinid = 0
                    'End If
                End With
                RaiseFindingAction(FindingActionMode.fscSetCurrentFinding, objFinding)
                If (myMouse.Button = Windows.Forms.MouseButtons.Right) Then
                    If (mbolAllowRowMenu) Then
                        ShowRowContextMenu(ugRow, objFinding)
                    End If
                End If

            ElseIf (Col = COL_FINDING) Then
                ' If no current encounter, don't allow this
                If (Not mbolAllowEdit) Then Exit Sub
                ' Added code to generate the event as if column 0 but without context menus.
                objFinding = New cFlowFinding
                myEventRow = ugRow
                With ugFlowSheet
                    'If (.Rows(Row).IsNode) Then
                    '    RowLevel = .Rows(Row).Node.Level
                    'Else
                    '    RowLevel = LVL_FINDING
                    'End If
                    'If (RowLevel = LVL_FINDING) Then
                    ' Medcinid stored in rowData CFlowRowData object
                    objRowData = ugRow.Tag
                    With objRowData
                        myCurrentMedcinid = .Medcinid
                        objFinding.Medcinid = myCurrentMedcinid
                        objFinding.Group = .Group
                        objFinding.SubGroup = .SubGroup
                    End With
                    'Else
                    '' Group and subgroup data stored in a cFlowSubGroupRow objec
                    'objRowData = .Rows(Row).UserData
                    'With objRowData
                    '    objFinding.Group = .Group
                    '    If (RowLevel = LVL_BODYSYS) Then
                    '        objFinding.SubGroup = .SubGroup
                    '    End If
                    'End With
                    'myCurrentMedcinid = 0
                    'End If
                    RaiseFindingAction(FindingActionMode.fscSetCurrentFinding, objFinding)
                End With
                With ugFlowSheet
                    ' If column COL_FINDING is not the current encounter, don't allow
                    If (e.Cell.Column.Tag = mstrEncounterTime) Then
                        If (Not IsNothing(e.Cell.Tag)) Then
                            ' Requesting selection of existing finding
                            objFinding = e.Cell.Tag
                            myCurrentMedcinid = objFinding.Medcinid
                            RaiseFindingAction(FindingActionMode.fscSelectFinding, objFinding)
                        End If
                    End If
                End With

            ElseIf (Col > COL_FINDING) Then
                ' For grid cells in cols > COL_FINDING and rows > 1,
                ' we allow citing to current encounter provisionally.
                ' If no current encounter, don't allow this
                If (Not mbolAllowEdit) Then Exit Sub
                ' If column COL_FINDING is not the current encounter, don't allow
                If (ugFlowSheet.DisplayLayout.Bands(0).Columns(COL_FINDING).Tag <> mstrEncounterTime) Then Exit Sub
                With ugFlowSheet
                    ' This is a cell other than current encounter; any data?
                    If (mbolCiteMode AndAlso Not e.Cell.Tag Is Nothing) Then
                        ' Only add it if the current encounter
                        ' is empty for this finding.
                        If (ugRow.Cells(COL_FINDING).Tag Is Nothing) Then
                            ' Requesting add of finding
                            myEventRow = ugRow
                            AddFlowFindingToEncounter(ugRow, Col)
                        End If
                    End If
                End With

            Else
                ' User clicked on COL_POS or COL_NEG for adding/editing/deleting
                ' If editing is not allowed, ignore this
                If (Not mbolAllowEdit) Then Exit Sub
                myEventRow = ugRow
                ProcessButtonAction(ugRow, Col)
            End If
        Catch ex As Exception

        End Try
    End Sub

    Private Sub ugFlowSheet_ClickCellButton(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.CellEventArgs) Handles ugFlowSheet.ClickCellButton
        Dim strInfo As String = String.Empty
        strInfo = "Cell: Row = " & e.Cell.Row.ListIndex & "; Col = " & e.Cell.Column.Index
        Dim oTag As Object = e.Cell.Tag
        If (Not oTag Is Nothing) Then
            strInfo = strInfo & "; Style=" & DirectCast(oTag, cCellData).DisplayStyle.ToString
        End If
        strInfo = strInfo & "; Mouse Button Clicked = " & myMouse.Button.ToString
        tssStatus.Text = strInfo
    End Sub

    Private Sub ugFlowSheet_InitializeGroupByRow(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.InitializeGroupByRowEventArgs) Handles ugFlowSheet.InitializeGroupByRow
        ' Do not need this as the DisplayLayout.Override.GroupByRowDescriptionMask is set to "[value]"
        'e.Row.Description = e.Row.Value ' This gets rid of the extraneous addition of child counts to the text
    End Sub

    Private Sub ugFlowSheet_InitializeLayout(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs) Handles ugFlowSheet.InitializeLayout
        'e.Layout.AutoFitStyle = AutoFitStyle.ResizeAllColumns
        'e.Layout.Override.RowSizing = RowSizing.AutoFree
        'e.Layout.Override.CellMultiLine = DefaultableBoolean.True

        'Dim ugGrid As UltraGrid = DirectCast(sender, UltraGrid)
        '' Turn on the fixed headers functionality. This will case the row selectors to be fixed.
        'e.Layout.UseFixedHeaders = True                             ' This allows setting up columns fixed from scrolling horizontally
        'e.Layout.ViewStyleBand = ViewStyleBand.OutlookGroupBy       ' This sets up for Outlook style grouping needed for group and subgroup
        ' These seem to have no effect
        Dim sType As String
        If (e.Layout.IsPrintLayout) Then
            sType = "IsPrintLayout"
        ElseIf (e.Layout.IsDisplayLayout) Then
            sType = "IsDisplayLayout"
        ElseIf (e.Layout.IsDrawing) Then
            sType = "IsDrawing"
        Else
            sType = "Other"
        End If
        Debug.WriteLine("ugFlowSheet_InitializeLayout: " & sType)
        e.Layout.RowConnectorStyle = RowConnectorStyle.Solid
        e.Layout.RowConnectorColor = Color.Red
        e.Layout.Override.BorderStyleCell = UIElementBorderStyle.Dotted
        e.Layout.Override.BorderStyleRow = UIElementBorderStyle.Dotted
        e.Layout.Override.CellDisplayStyle = UltraWinGrid.CellDisplayStyle.Default

        e.Layout.Bands.Layout.RowConnectorStyle = RowConnectorStyle.Dotted
    End Sub

    Private Sub ugFlowSheet_InitializePrint(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.CancelablePrintEventArgs) Handles ugFlowSheet.InitializePrint
        Debug.WriteLine("ugFlowSheet_InitializePrint")
    End Sub

    Private Sub ugFlowSheet_InitializePrintPreview(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinGrid.CancelablePrintPreviewEventArgs) Handles ugFlowSheet.InitializePrintPreview
        Debug.WriteLine("ugFlowSheet_InitializePrintPreview")
    End Sub

    Private Sub ugFlowSheet_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ugFlowSheet.MouseDown
        myMouse = e
        Dim gbRow As Boolean = False
        Dim ugUIElement As UIElement = ugFlowSheet.DisplayLayout.UIElement.ElementFromPoint(myMouse.Location)
        Dim ugRow As UltraGridRow = ugUIElement.GetContext(GetType(UltraGridGroupByRow))
        If Not ugRow Is Nothing Then gbRow = True
        '' Get the row that contains this element.
        'Dim row As UltraGridRow = ugUIElement.GetContext(GetType(UltraGridRow))
        '' A column header will not be grid row type
        'If (Not row Is Nothing) Then Exit Sub
        ' Retrieve the Column from the UIElement
        Dim ugColumn As UltraGridColumn = ugUIElement.GetContext(GetType(UltraGridColumn))
        ' If not a column, then exit
        If (Not gbRow AndAlso ugColumn Is Nothing) Then
            'Debug.WriteLine("MouseDown: Not a column")
            Exit Sub
        End If
        'If (gbRow) Then
        '    Debug.WriteLine("MouseDown: Is a GroupBy row")
        'Else
        '    Debug.WriteLine("MouseDown: Is a column")
        'End If
        ' Switch to extended selection on GroupBy rows or finding header column
        If (gbRow OrElse ugColumn.Index <= COL_HEAD) Then
            'With ugFlowSheet.DisplayLayout
            '    If (.Override.CellClickAction = CellClickAction.CellSelect) Then
            '        If (Not gbRow) Then
            '            ugRow = ugUIElement.GetContext(GetType(UltraGridRow))
            '        End If
            '        If (Not ugRow Is Nothing) Then
            '            ugRow.Selected = True
            '            'Debug.WriteLine("Just selected row " & ugRow.ListIndex)
            '            'ugRow.Refresh()
            '        End If
            '    End If
            '    .Override.CellClickAction = CellClickAction.RowSelect  ' We do not go into edit mode
            '    .Override.SelectTypeCell = SelectType.Extended
            '    .Override.SelectTypeRow = SelectType.Extended
            '    .Override.SelectTypeGroupByRow = SelectType.Extended    ' Keeps GroupBy rows (group, subgroup) from selecting
            'End With
        Else
            With ugFlowSheet.DisplayLayout
                ' If we just switched from extended select to cell select, deselect any selected rows
                If (.Override.CellClickAction = CellClickAction.RowSelect) Then
                    Dim oSelected As Selected = ugFlowSheet.Selected
                    If (oSelected.Rows.Count > 0) Then
                        oSelected.Rows.Clear()
                    End If
                End If
                .Override.CellClickAction = CellClickAction.CellSelect  ' We do not go into edit mode
                .Override.SelectTypeCell = SelectType.Single
                .Override.SelectTypeRow = SelectType.None
                .Override.SelectTypeGroupByRow = SelectType.None    ' Keeps GroupBy rows (group, subgroup) from selecting
            End With
        End If
    End Sub


    Private Sub ugFlowSheet_MouseEnterElement(ByVal sender As Object, ByVal e As Infragistics.Win.UIElementEventArgs) Handles ugFlowSheet.MouseEnterElement
        ' Retrieve the Column from the UIElement
        Dim ugColumn As UltraGridColumn = e.Element.GetContext(GetType(UltraGridColumn))
        ' If not a column, then exit
        If (ugColumn Is Nothing) Then GoTo ClearStatus
        ' We use this only for history columns; cells of fixed columns have a different Tag object
        If (ugColumn.Index <= COL_FINDING) Then GoTo ClearStatus

        Dim ugCell As UltraGridCell = e.Element.GetContext(GetType(UltraGridCell))
        If (ugCell Is Nothing) Then GoTo ClearStatus
        If (Not ugCell.IsDataCell) Then GoTo ClearStatus
        If (ugCell.Tag Is Nothing) Then GoTo ClearStatus
        Dim oCell As cCellData = ugCell.Tag
        ' ListIndex is the actual row of the bound data; Index is the row in the child band
        Dim strInfo As String = "Mouse entered Cell: (R,C)=(" & ugCell.Row.ListIndex & "," & ugColumn.Index & ")"
        tssStatus.Text = strInfo
        Exit Sub
ClearStatus:
        tssStatus.Text = String.Empty
    End Sub

    Private Sub ugFlowSheet_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ugFlowSheet.MouseUp
        With ugFlowSheet.DisplayLayout
            If (.Override.CellClickAction = CellClickAction.CellSelect) Then
                .Override.CellClickAction = CellClickAction.RowSelect  ' We do not go into edit mode
                .Override.SelectTypeCell = SelectType.Extended
                .Override.SelectTypeRow = SelectType.Extended
                .Override.SelectTypeGroupByRow = SelectType.Extended    ' Keeps GroupBy rows (group, subgroup) from selecting
            End If
        End With
    End Sub


    'The SelectedCellForeColorDrawFilterClass handles removing the Selected color and 
    Private Class CellDrawFilter
        Implements IUIElementDrawFilter 'Implement the IUIElementDrawFilter interface

#Region " GetPhasesToFilter"

        Public Function GetPhasesToFilter(ByRef drawParams As Infragistics.Win.UIElementDrawParams) As Infragistics.Win.DrawPhase Implements Infragistics.Win.IUIElementDrawFilter.GetPhasesToFilter
            'In order to handle the drawing, we need to override the drawing of the 
            'TextUIElement inside the cell. So first, we check to see if the Element
            'is a TextUIElementBase. Second, we check to see if it's context is a cell.
            'If so, we return the BeforeDrawForeGround phase, so we can override it. 
            If (TypeOf drawParams.Element Is TextUIElementBase) AndAlso (Not drawParams.Element.GetContext(GetType(UltraGridCell)) Is Nothing) Then
                Return DrawPhase.BeforeDrawForeground + DrawPhase.BeforeDrawFocus
            Else
                Return DrawPhase.BeforeDrawFocus
            End If
        End Function
        ' GetPhasesToFilter
#End Region

#Region " DrawElement"

        Public Function DrawElement(ByVal drawPhase As Infragistics.Win.DrawPhase, ByRef drawParams As Infragistics.Win.UIElementDrawParams) As Boolean Implements Infragistics.Win.IUIElementDrawFilter.DrawElement
            'Dim a variable to hold the cell 
            Dim aCell As UltraGridCell

            Select Case drawPhase
                Case drawPhase.BeforeDrawFocus
                    Return True     ' Inhibit drawing of focus rectangle
                Case drawPhase.BeforeDrawForeground

                    'Get a reference to the cell
                    aCell = drawParams.Element.GetContext(GetType(UltraGridCell))
                    If (Not aCell Is Nothing) Then
                        Dim brush As Brush
                        Dim sFormat As New System.Drawing.StringFormat
                        With sFormat
                            .Alignment = StringAlignment.Near
                            .LineAlignment = StringAlignment.Center
                            .Trimming = StringTrimming.EllipsisCharacter
                            .FormatFlags = StringFormatFlags.NoWrap
                        End With
                        If (aCell.Column.Index = COL_HEAD) Then
                            If (aCell.HasAppearance) Then
                                brush = drawParams.CreateSolidBrush(aCell.Appearance.ForeColor)
                            Else
                                brush = drawParams.CreateSolidBrush(aCell.Row.Appearance.ForeColor)
                            End If
                            Try
                                With drawParams.Element.Rect  ' was .InvalidRect
                                    drawParams.Graphics.DrawString(aCell.Value, drawParams.Font, brush, RectangleF.FromLTRB(.Left, .Top, .Right, .Bottom), sFormat)
                                End With

                            Catch ex As Exception
                                Debug.WriteLine(ex.Message)
                            End Try
                            brush.Dispose()
                            Return True ' Draw nothing
                        End If
                        If (aCell.Column.Index <= COL_FINDING) Then Return False
                        If (aCell.Tag Is Nothing) Then Return False
                        Dim oCellData As cCellData = aCell.Tag
                        'If (oCellData.CellData.Count <= 1) Then Return False
                        Dim Lines() As String = aCell.Text.Split(vbLf)
                        Dim oRowData As cFlowRowData = DirectCast(aCell.Row.Tag, cFlowRowData)
                        Dim Interval As Integer = aCell.Row.Height \ oRowData.LineCount
                        Dim ptTop, ptLeft, ptRight, ptBottom As Single
                        With drawParams.Element.Rect  ' was .InvalidRect
                            ptLeft = .Left
                            ptTop = .Top
                            ptRight = .Right
                            ptBottom = ptTop + Interval
                        End With
                        If (Lines.Count < oRowData.LineCount) Then
                            Dim Offset As Single = (oRowData.LineCount - Lines.Count) * 0.5 * Interval
                            ptTop += Offset
                            ptBottom += Offset
                        End If
                        If (aCell.HasAppearance) Then
                            brush = drawParams.CreateSolidBrush(aCell.Appearance.ForeColor)
                        Else
                            brush = drawParams.CreateSolidBrush(aCell.Row.Appearance.ForeColor)
                        End If
                        Dim loRect As RectangleF
                        Dim strLine As String
                        Try
                            For Each strLine In Lines
                                loRect = RectangleF.FromLTRB(ptLeft, ptTop, ptRight, ptBottom)
                                drawParams.Graphics.DrawString(strLine, drawParams.Font, brush, loRect, sFormat)
                                ptTop += Interval
                                ptBottom += Interval
                            Next
                        Catch ex As Exception
                            Debug.WriteLine(ex.Message)
                        End Try
                        brush.Dispose()
                        Return True ' Draw nothing
                    End If
            End Select
        End Function
        ' DrawElement
#End Region
    End Class

    Protected Overrides Sub Finalize()
        MyBase.Finalize()
    End Sub

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'initializeinternaldata()

    End Sub

    Private Sub btnPrintGrid_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrintGrid.Click
        Dim Dlg As Printing.UltraPrintPreviewDialog = New Printing.UltraPrintPreviewDialog
        Dim GridDoc As UltraGridPrintDocument = New UltraGridPrintDocument
        Try
            GridDoc.Grid = ugFlowSheet
            Dlg.Document = GridDoc
            Dlg.ShowDialog(Me)
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

End Class
