
''' <summary>
''' This enumerates the possible doctor types
''' </summary>
''' <remarks></remarks>
Public Enum MedDoctorType As Integer
    ''' <summary>Value = 0; No doctor is specified.</summary>
    medDocNone = 0
    ''' <summary>Value = 1; This specifies the doctor entering data.</summary>
    medDocEntering = 1
End Enum

''' -----------------------------------------------------------------------------
''' Project	 : Objects
''' Class	 : Objects.MedChart
''' 
''' -----------------------------------------------------------------------------
''' <summary>
''' This object is used to pass a chart finding. 
''' </summary>
''' <remarks>
''' This object contains all of the various properties of a finding that is entered 
''' into a patient's chart. Only those properties used in the Medicomp namespace
''' are present.
''' </remarks>
''' <history>
''' 	[roy]	2/18/2005	Created
'''     [roy]   7/06/2006   Added Specifier property
'''     [roy]   4/29/2009   Added UserSubGroup property
''' </history>
''' -----------------------------------------------------------------------------
Public Class MedChart
    'Private mydValue As Double = Double.NaN
    Private myRangeNormalLow As Double
    Private myRangeNormalHigh As Double
    Private myEditDateTime As Date
    Private myUniqueid As Integer
    Private myMedcinid As Integer
    Private myReferid As Integer                ' referred to docid
    Private myBlobid As Integer                 ' id of attachment
    Private myRxid As Integer
    Private myEnterDocID As Integer             ' ** Superceded by DoctorReferences
    Private mySubGroup As Integer               ' Finding's subgroup (body system, usually)
    Private myCodes As String                   ' Assigned codelink codes
    Private myPrefix As String
    Private myModifier As String
    Private myResult As String
    Private myStatus As String
    Private myOnset As String
    Private myDuration As String
    Private myValue As String
    Private myUnit As String
    Private myNote As String
    Private myEpisode As String                 ' Note: this field not supported in demo
    Private mySblob As String
    Private myReferenceID As String
    Private myQualifierLink As String           ' qualifier link
    Private mySpecifier As String
    Private myTag As String
    Private myText As String                    ' Transcription of this object
    Private myTiming As String ' Order timing
    Private myItemData As String
    Private myQualifierLinkSequence As Integer  ' target of qualifier link
    Private mySeq As Short
    Private myGroup As Short
    Private myMedcinGroup As Short       ' Narrative Group assigned by Medcin
    Private myChapter As Short                  ' App assigned chapter number
    Private myRangeScale As Short
    Private myUserSubGroup As Short             ' Application assigned subgroup for miscellaneous group
    Private myFlag As Short
    Private myRxObj As MedPrescription
    Private myDoctorReferences As MedDoctorReferences
    Private myChanged As Boolean
    Private myApp As Object                     ' For application defined

    ''' <summary>
    ''' This function is used to obtain the chart value as a double
    ''' </summary>
    ''' <returns>The value property as converted to a double.</returns>
    ''' <remarks>For value strings of the form 'lbs#oz', the value returned
    ''' will be equal to lbs + (oz/16). If the value property starts with
    ''' a less than or greater than character, the value returned will be
    ''' equal to what ever numeric value follows that character. For any other
    ''' value syntax, the number returned will be the numeric value. Note that 
    ''' DoubleTryParse() is used for conversion; thus, any malformed string
    ''' is returned as 0.</remarks>
    Public Function ConvertValue() As Double
        Dim dValue As Double = 0.0
        If (myValue Is Nothing OrElse myValue.Length = 0) Then Return dValue
        Dim iPos As Integer = myValue.IndexOf("#")
        If (iPos >= 0 AndAlso myValue.Length > iPos + 1 AndAlso myUnit = "lbs") Then
            Dim dValue1 As Double
            Double.TryParse(myValue.Substring(0, iPos), dValue)
            Double.TryParse(myValue.Substring(iPos + 1), dValue1)
            dValue += (dValue1 / 16)
        ElseIf (myValue.StartsWith("<") OrElse myValue.StartsWith(">")) Then
            ' Ignore the less than or greater than character for the dValue
            Double.TryParse(myValue.Substring(1), dValue)
        Else
            Double.TryParse(myValue, dValue)
        End If
        Return dValue
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to obtain a clone of the object
    ''' </summary>
    ''' <returns>Returns a new objec with this object's data.</returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Function Clone() As MedChart
        ' We need this internal for ClinicalSummary
        Dim newObject As New MedChart
        Dim oDocref As MedDoctorReference
        With newObject
            .Blobid = myBlobid
            .Changed = myChanged
            .Chapter = myChapter
            .Codes = myCodes
            .Duration = myDuration
            .EditDateTime = myEditDateTime
            .Episode = myEpisode
            .Flag = myFlag
            .Group = myGroup
            .ItemData = myItemData
            .Medcinid = myMedcinid
            .MedcinGroup = myMedcinGroup
            .Modifier = myModifier
            .Note = myNote
            .Onset = myOnset
            .Prefix = myPrefix
            .QualifierLink = myQualifierLink
            .QualifierLinkSequence = myQualifierLinkSequence
            .RangeNormalHigh = myRangeNormalHigh
            .RangeNormalLow = myRangeNormalLow
            .RangeScale = myRangeScale
            .ReferenceID = myReferenceID
            .Referid = myReferid
            .Result = myResult
            If (myRxObj Is Nothing) Then
                .RxObj = Nothing
            Else
                .RxObj = myRxObj.Clone
            End If
            .SBlob = mySblob
            .Seq = mySeq
            .Specifier = mySpecifier
            .Status = myStatus
            .SubGroup = mySubGroup
            .Timing = myTiming
            .Text = myText
            .Uniqueid = myUniqueid
            .Unit = myUnit
            .UserSubGroup = myUserSubGroup
            .Value = myValue
            .Tag = myTag
            ' If this has a non-empty collection
            ' then clone it
            If (myDoctorReferences.Count > 0) Then
                For Each oDocref In myDoctorReferences
                    .DoctorReferences.Add(oDocref.Code, oDocref.Docid)
                Next oDocref
            End If
        End With
        Return newObject
    End Function

    Public Overrides Function ToString() As String
        Dim sb As New System.Text.StringBuilder("U=")
        sb.Append(Me.Uniqueid.ToString)
        sb.Append(",M=")
        sb.Append(Me.Medcinid.ToString)
        sb.Append(",F=")
        sb.Append(Me.Flag.ToString)
        sb.Append(",G=")
        sb.Append(Me.Group.ToString)
        sb.Append(",MG=")
        sb.Append(Me.MedcinGroup.ToString)
        sb.Append(",P=")
        sb.Append(Me.Prefix)
        sb.Append(",R=")
        sb.Append(Me.Result)
        sb.Append(",N=[")
        sb.Append(Me.Note)
        sb.Append("]")
        Return sb.ToString
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The App property is reserved strictly for use by your application. Medicomp
    ''' GUI makes no use of it.
    ''' </summary>
    ''' <value>An application defined object.</value>
    ''' <remarks>
    ''' Note that the Clone method does not duplicate this property.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/23/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property App() As Object
        Get
            Return myApp
        End Get
        Set(ByVal Value As Object)
            myApp = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This can be used to indicate that one or more fields of the object have been changed.
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Changed() As Boolean
        Get
            Return myChanged
        End Get
        Set(ByVal Value As Boolean)
            myChanged = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is a unique identifier for sBlob; currently used in this implementation 
    ''' for attachments to findings (image annotations).
    ''' </summary>
    ''' <value>The unique identiier.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Blobid() As Integer
        Get
            Return myBlobid
        End Get
        Set(ByVal Value As Integer)
            myBlobid = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Application assigned chapter number used to divide an encounter into chapters
    ''' </summary>
    ''' <value>A chapter number from 0 to n.</value>
    ''' <remarks>
    ''' The chapter number is a short value; the first chapter is numbered 0. Unless 
    ''' chapters are assigned, all findings added to the encounter are assigned to 
    ''' chapter 0. Thus, your application does not have to use chapters. The chapter 
    ''' number is assigned automatically by the Medcin server when a finding is newly 
    ''' added to the encounter via AddNarrate or AddNarrateEx. The chapter number applied 
    ''' is established by the  application using the SetChapter API.
    ''' <para>The GUI modules do not maintain the current chapter number established by
    ''' your application; thus, the application should set this property (if other than 0)
    ''' when the AfterChartAction event is raised.</para>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	7/19/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Chapter() As Short
        Get
            Return myChapter
        End Get
        Set(ByVal Value As Short)
            myChapter = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This may contain one or more unique MedcinDoctorReference objects.
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' These are used to delineate a DoctorID associated with a particular function 
    ''' of this Chart record. Currently, one reference code is supported 
    ''' (medDocEntering as discussed in the SDK manual) used to set the DoctorID 
    ''' when the doctor entering the data is not the DoctorID as specified in the 
    ''' MedEncounter object for the encounter.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property DoctorReferences() As MedDoctorReferences
        Get
            Return myDoctorReferences
        End Get
        Set(ByVal Value As MedDoctorReferences)
            myDoctorReferences = Value
        End Set
    End Property

    '''' <summary>
    '''' This property can be used to store the numeric part of the Value property as a number
    '''' </summary>
    '''' <value>The numerical part of the Value property (see ConvertValue)</value>
    '''' <returns>The numerical value passed to the object is returned.</returns>
    '''' <remarks>If no value was Put, then the Get will return Double.NaN</remarks>
    'Public Property dValue() As Double
    '    Get
    '        Return mydValue
    '    End Get
    '    Set(ByVal value As Double)
    '        mydValue = value
    '    End Set
    'End Property

    ''' <summary>
    ''' This property contains codelink codes assigned during user session
    ''' </summary>
    ''' <value>Contains an encoded string, possibly of multiple codes</value>
    ''' <returns>The codes assigned are returned.</returns>
    ''' <remarks>The syntax of a A Codes string is as follows:
    ''' <para>codetype,code[;codetype,code]</para>
    ''' <para>where codetype is one of the numbered codetype's (see CodeList table in codelink.mdb)
    ''' or the Medicomp.Codelinks enumeration,
    ''' and code is the Medicomp-assigned code mapping.</para></remarks>
    Public Property Codes() As String
        Get
            Return myCodes
        End Get
        Set(ByVal value As String)
            myCodes = value
        End Set
    End Property
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the HL7 timing component assigned for special orders.
    ''' </summary>
    ''' <value>The timing string.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Timing() As String
        Get
            Return myTiming
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myTiming = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is available for application defined usage.
    ''' </summary>
    ''' <value>A string containing the data.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property ItemData() As String
        Get
            Return myItemData
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myItemData = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This contains the last date and time that a property of this object was changed. 
    ''' It is up to the application to maintain this property.
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property EditDateTime() As Date
        Get
            Return myEditDateTime
        End Get
        Set(ByVal Value As Date)
            myEditDateTime = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the unique identifier required for data entry
    ''' </summary>
    ''' <value>The uniqueID assigned.</value>
    ''' <remarks>
    ''' Every finding in an encounter requires a unique identifier that is assigned
    ''' by your application. When the user adds a finding during data entry, your
    ''' application typically populates this property with a unique number during the
    ''' ValidateChart event.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Uniqueid() As Integer
        Get
            Return myUniqueid
        End Get
        Set(ByVal Value As Integer)
            myUniqueid = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the permanent Medcin identification number of the finding.
    ''' </summary>
    ''' <value>The Medcin finding number.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Medcinid() As Integer
        Get
            Return myMedcinid
        End Get
        Set(ByVal Value As Integer)
            myMedcinid = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the Medcin prefix qualifier code string.
    ''' </summary>
    ''' <value>The prefix code.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Prefix() As String
        Get
            Return myPrefix
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myPrefix = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the Medcin modifier qualifier code string.
    ''' </summary>
    ''' <value>The modifier code.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Modifier() As String
        Get
            Return myModifier
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myModifier = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the Medcin result qualifier code string.
    ''' </summary>
    ''' <value>The result code.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Result() As String
        Get
            Return myResult
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myResult = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is a semicolon delimited list of codespec lists.
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' A codespec list is a comma-delimited list of codes. The specifier property
    ''' can contain more than one list of codes, with each list delimited
    ''' using a semicolon. The specifier starts with a field indicating the codetype
    ''' of the code used to obtain the lists (ex: 23;A,B,D;AB,C,DA). The specifier is used in
    ''' conjunction with a DSM-IV code to add additional uncoded textual content to 
    ''' commonly used in a finding's narration. The lists would be in sequence with
    ''' a corresponding codespec record for the DSM-IV code in question.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	7/6/2006	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Specifier() As String
        Get
            If (mySpecifier Is Nothing) Then
                Return ""
            Else
                Return mySpecifier
            End If
        End Get
        Set(ByVal Value As String)
            mySpecifier = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the Medcin status qualifier code string.
    ''' </summary>
    ''' <value>The status code.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Status() As String
        Get
            Return myStatus
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myStatus = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the sub-group that is assigned by Medcin
    ''' </summary>
    ''' <value>The sub-group, typically a body system or other grouping below narrative group..</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	3/21/2006	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property SubGroup() As Integer
        Get
            Return mySubGroup
        End Get
        Set(ByVal Value As Integer)
            mySubGroup = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the onset of the observation in Medcin date format, (YYYYMMDDHHNN).
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Onset() As String
        Get
            Return myOnset
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myOnset = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the duration of the finding maintained as a unit of time (a number 
    ''' followed by a unit of time specification character).
    ''' </summary>
    ''' <value>The duration as a string.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Duration() As String
        Get
            Return myDuration
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myDuration = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the measurement value assigned.
    ''' </summary>
    ''' <value>The value as a string.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Value() As String
        Get
            Return myValue
        End Get
        Set(ByVal ChartValue As String)
            If (Not Value Is Nothing) Then myValue = ChartValue
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the unit for the value field.
    ''' </summary>
    ''' <value>The unit string.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Unit() As String
        Get
            Return myUnit
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myUnit = Value
        End Set
    End Property

    ''' <summary>
    ''' Property for assigning a subgroup number when the group is one of the miscellaneous groups
    ''' </summary>
    ''' <value>A number from 0 to n; 0 implies remove the subgroup number.</value>
    ''' <returns>The current value of the subgroup number assigned by the application. Default is 0.</returns>
    ''' <remarks>Do not confuse this property with the SubGroup property which has the subgroup value assigned by the Medcin server.</remarks>
    Public Property UserSubGroup() As Short
        Get
            Return myUserSubGroup
        End Get
        Set(ByVal value As Short)
            myUserSubGroup = value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the calculated range scale.
    ''' </summary>
    ''' <value>A scale value (0-7) usually recovered from the Medcin server's CalcScale API.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property RangeScale() As Short
        Get
            Return myRangeScale
        End Get
        Set(ByVal Value As Short)
            myRangeScale = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the normal low value of the range.
    ''' </summary>
    ''' <value>The normal low, typically recovered from the Medcin server.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property RangeNormalLow() As Double
        Get
            Return myRangeNormalLow
        End Get
        Set(ByVal Value As Double)
            myRangeNormalLow = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the normal high value of the range.
    ''' </summary>
    ''' <value>The normal high, typically recovered from the Medcin server.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property RangeNormalHigh() As Double
        Get
            Return myRangeNormalHigh
        End Get
        Set(ByVal Value As Double)
            myRangeNormalHigh = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the id number assigned to a provider who is referred the patient for this finding.
    ''' </summary>
    ''' <value>The docid of the referring provider</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Referid() As Integer
        Get
            Return myReferid
        End Get
        Set(ByVal Value As Integer)
            myReferid = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This stores a bit structure used by the GUI objects to further 
    ''' classify the finding. 
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' Bit assignments are assigned in the ChartFlags enumeration that is defined in
    ''' the Medicomp.Constants namespace. This namespace is found in globals.dll.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Flag() As Short
        Get
            Return myFlag
        End Get
        Set(ByVal Value As Short)
            myFlag = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This contains the free-text note attached to the finding.
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Note() As String
        Get
            Return myNote
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myNote = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is a string that contains a packed version of the episode fields.
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' The format is 'expression|FreqFm|FreqTo|DurFm|DurTo'. Expression is 'F' for 
    ''' frequency or 'I' for interval. Each additional field is formatted as a series 
    ''' of digits followed by a time unit character ('N' minute, 'H' hour, 'D' day, 
    ''' 'W' week, 'M' month, 'Y' year). The string is converted to presentation 
    ''' format for narration.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Episode() As String
        Get
            Return myEpisode
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myEpisode = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This stores the docid of the provider entering the finding.
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' Note that this is deprecated. Use the DoctorReferences property instead.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property EnterDocID() As Integer
        Get
            Return myEnterDocID
        End Get
        Set(ByVal Value As Integer)
            myEnterDocID = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This contains an optional attached string (image annotation)
    ''' </summary>
    ''' <value>The string containing the data.</value>
    ''' <remarks>
    ''' The Medicomp GUI supports an image annotation available for each finding.
    ''' This property is used to store the annotation string generated by the imaging
    ''' facility.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property SBlob() As String
        Get
            Return mySblob
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then mySblob = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the reference identification for tests and therapies
    ''' </summary>
    ''' <value></value>
    ''' <remarks>
    ''' This should be a unique identification string that relates results to the
    ''' finding that ordered the test or therapy.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property ReferenceID() As String
        Get
            Return myReferenceID
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myReferenceID = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the code string assigned for linking two findings, or the link group 
    ''' code assigned when grouping two or more related findings.
    ''' </summary>
    ''' <value>The link code.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property QualifierLink() As String
        Get
            Return myQualifierLink
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myQualifierLink = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This property is used to persist the user's custom presentation sequence of
    ''' assessment findings in the narrative
    ''' </summary>
    ''' <value>The order of presentation: 1, 2, 3, etc. Defaults to 0.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	5/4/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Seq() As Short
        Get
            Return mySeq
        End Get
        Set(ByVal Value As Short)
            mySeq = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the UniqueID number of the linked finding.
    ''' </summary>
    ''' <value>The UniqueID of the other finding of the pair.</value>
    ''' <remarks>
    ''' When two findings (A and B) are linked together, A stores the UniqueID of B
    ''' in this property and B stores the UniqueID of A in this property.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property QualifierLinkSequence() As Integer
        Get
            Return myQualifierLinkSequence
        End Get
        Set(ByVal Value As Integer)
            myQualifierLinkSequence = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is the prescription object, if the finding has an associated prescription.
    ''' </summary>
    ''' <value>The MedPrescription object.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property RxObj() As MedPrescription
        Get
            Return myRxObj
        End Get
        Set(ByVal Value As MedPrescription)
            myRxObj = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This stores the Narrative group if assigned by the application, otherwise the value stored is 0.
    ''' </summary>
    ''' <value>The narrative group.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Group() As Short
        Get
            Return myGroup
        End Get
        Set(ByVal Value As Short)
            myGroup = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This stores the narrative group assigned by the Medcin narrative engine
    ''' </summary>
    ''' <value>The Medcin assigned narrative group</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	3/21/2006	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property MedcinGroup() As Short
        Get
            Return myMedcinGroup
        End Get
        Set(ByVal Value As Short)
            myMedcinGroup = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used for the string itemtag.
    ''' </summary>
    ''' <value>A string to be used as the itemtag parameter of the AddItemtag API</value>
    ''' <remarks>
    ''' The Medcin narrative provides for a string tag to be applied to each finding
    ''' in an encounter. This property is used to propogate your tag to that parameter.
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Tag() As String
        Get
            Return myTag
        End Get
        Set(ByVal Value As String)
            If (Not Value Is Nothing) Then myTag = Value
        End Set
    End Property

    ''' <summary>
    ''' This property is used to contain a transcription of the finding
    ''' </summary>
    ''' <value>The transcription of the finding.</value>
    ''' <returns>The Trancription of the finding is returned.</returns>
    ''' <remarks></remarks>
    Public Property Text() As String
        Get
            Return myText
        End Get
        Set(ByVal value As String)
            myText = value
        End Set
    End Property

    ''' <summary>
    ''' This creates a new class object.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        myPrefix = PREFIX_BLANK
        myModifier = String.Empty
        myResult = RESULT_BLANK
        myStatus = String.Empty
        myOnset = String.Empty
        myDuration = String.Empty
        myValue = String.Empty
        myUnit = String.Empty
        myNote = String.Empty
        myEpisode = String.Empty
        mySblob = String.Empty
        myReferenceID = String.Empty
        myQualifierLink = String.Empty
        myTag = String.Empty
        myText = String.Empty
        myTiming = String.Empty
        myCodes = String.Empty
        myDoctorReferences = New MedDoctorReferences
        myChanged = False
    End Sub
End Class

''' -----------------------------------------------------------------------------
''' Project	 : Objects
''' Class	 : Objects.MedDoctorReference
''' 
''' -----------------------------------------------------------------------------
''' <summary>
''' This class is used to identiy a doctor reference
''' </summary>
''' <remarks>
''' </remarks>
''' <history>
''' 	[roy]	2/22/2005	Created
''' </history>
''' -----------------------------------------------------------------------------
Public Class MedDoctorReference
    Private myCode As Short
    Private myDocid As Integer

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This identifies the identification number of the provider.
    ''' </summary>
    ''' <value>The provider's id.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Docid() As Integer
        Get
            Return myDocid
        End Get
        Set(ByVal Value As Integer)
            myDocid = Value
        End Set
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to specify the provider type.
    ''' </summary>
    ''' <value>One of the Medicomp.DocCodes members.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Property Code() As Short
        Get
            Return myCode
        End Get
        Set(ByVal Value As Short)
            myCode = Value
        End Set
    End Property
End Class

''' -----------------------------------------------------------------------------
''' Project	 : Objects
''' Class	 : Objects.MedDoctorReferences
''' 
''' -----------------------------------------------------------------------------
''' <summary>
''' This class is used to contain one or more unique DoctorReference objects.
''' </summary>
''' <remarks>
''' </remarks>
''' <history>
''' 	[roy]	2/22/2005	Created
''' </history>
''' -----------------------------------------------------------------------------
Public Class MedDoctorReferences
    Implements System.Collections.IEnumerable

    Private mCol As Collection

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Procedure to clear the collection
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Sub Clear()
        While (mCol.Count() > 0)
            mCol.Remove(1)
        End While
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Function to add a new reference to the collection.
    ''' </summary>
    ''' <param name="Code">The provider type code.</param>
    ''' <param name="Docid">The unique identifier of the provider.</param>
    ''' <returns>A reference to the new MedDoctorObject created.</returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Function Add(ByVal Code As Short, ByVal Docid As Integer) As MedDoctorReference
        'create a new object
        Dim objNewMember As MedDoctorReference
        objNewMember = New MedDoctorReference

        'set the properties passed into the method
        objNewMember.Code = Code
        objNewMember.Docid = Docid
        mCol.Add(objNewMember, "K" & CStr(Code))

        'return the object created
        Add = objNewMember
        objNewMember = Nothing
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This provides an alternative means of adding a MedDoctorReference.
    ''' </summary>
    ''' <param name="objDocRef">The MedDoctorReference.</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Sub AddDoctorReference(ByVal objDocRef As MedDoctorReference)
        ' Procedure adds the passed object using our key
        mCol.Add(objDocRef, "K" & CStr(objDocRef.Code))
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to obtain one member of the collection by its index or key.
    ''' </summary>
    ''' <param name="IndexKey">The index or key of the item.</param>
    ''' <value>The MedDoctorReference object requested, or Nothing if not available.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Default Public ReadOnly Property Item(ByVal IndexKey As Object) As MedDoctorReference
        Get
            Try
                Return mCol.Item(IndexKey)
            Catch
                Return Nothing
            End Try
        End Get
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to obtain a count of the items in the collection.
    ''' </summary>
    ''' <value>The count of items.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public ReadOnly Property Count() As Integer
        Get
            Count = mCol.Count()
        End Get
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to recover the enumerator of the collection.
    ''' </summary>
    ''' <returns>This returns the enumerator of the collection.</returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        GetEnumerator = mCol.GetEnumerator
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This allows you to remove a MedDoctorReference from the collection.
    ''' </summary>
    ''' <param name="IndexKey">The index or key of the item.</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Sub Remove(ByVal IndexKey As Object)
        mCol.Remove(IndexKey)
    End Sub

    ''' <summary>
    ''' This creates a new object.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.New()
        'creates the collection when this class is created
        mCol = New Collection
    End Sub

    ''' <summary>
    ''' This finalizes the object
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overrides Sub Finalize()
        mCol = Nothing
        MyBase.Finalize()
    End Sub
End Class

''' -----------------------------------------------------------------------------
''' Project	 : Objects
''' Class	 : Objects.MedCharts
''' 
''' -----------------------------------------------------------------------------
''' <summary>
''' This class is used to contain a collection of MedChart objects.
''' </summary>
''' <remarks>
''' </remarks>
''' <history>
''' 	[roy]	2/22/2005	Created
''' </history>
''' -----------------------------------------------------------------------------
Public Class MedCharts
    Implements System.Collections.IEnumerable

    Private mCol As Collection
    Private myChanged As Boolean

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to remove all items in the collection.
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Sub Clear()
        Try
            If (Not mCol Is Nothing) Then
                While (mCol.Count() > 0)
                    mCol.Remove(1)
                End While
            End If
        Catch ex As Exception
            'MsgBox("MedCharts.Clear:" & ex.Message, MsgBoxStyle.Critical)
        End Try
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to add a MedChart object to the collection.
    ''' </summary>
    ''' <param name="oChart">The MedChart to add.</param>
    ''' <param name="sKey">Optional. A unique String expression that specifies a key 
    ''' string that can be used instead of a positional index to access a member of 
    ''' the collection.</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Sub AddChart(ByVal oChart As MedChart, Optional ByVal sKey As String = "")
        If Len(sKey) = 0 Then
            mCol.Add(oChart)
        Else
            mCol.Add(oChart, sKey)
        End If
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to insert a MedChart object after a given object.
    ''' </summary>
    ''' <param name="oChart">The MedChart object to add.</param>
    ''' <param name="oAfter">An expression that specifies a relative position in the 
    ''' collection. The member to be added is placed in the collection after the member 
    ''' identified by the After argument. If After is a numeric expression, it must 
    ''' be a number from 1 to the value of the collection's Count property. If After 
    ''' is a String expression, it must correspond to the key string specified when 
    ''' the member referred to was added to the collection.</param>
    ''' <param name="sKey">Optional. A unique String expression that specifies a key 
    ''' string that can be used instead of a positional index to access a member of 
    ''' the collection.</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Sub AddChartAfter(ByVal oChart As MedChart, ByVal oAfter As Object, Optional ByVal sKey As String = "")
        If Len(sKey) = 0 Then
            mCol.Add(oChart, , , oAfter)
        Else
            mCol.Add(oChart, sKey, , oAfter)
        End If
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to insert a MedChart object before a given object.
    ''' </summary>
    ''' <param name="oChart">This is the reference to the chart object to insert.</param>
    ''' <param name="oBefore">Optional. An expression that specifies a relative position 
    ''' in the collection. The member to be added is placed in the collection before the 
    ''' member identified by the Before argument. If Before is a numeric expression, 
    ''' it must be a number from 1 to the value of the collection's Count property. 
    ''' If Before is a String expression, it must correspond to the key string specified 
    ''' when the member being referred to was added to the collection.</param>
    ''' <param name="sKey">Optional. A unique String expression that specifies a key string 
    ''' that can be used instead of a positional index to access a member of the collection.</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Sub AddChartBefore(ByVal oChart As MedChart, ByVal oBefore As Object, Optional ByVal sKey As String = "")
        If Len(sKey) = 0 Then
            mCol.Add(oChart, , oBefore)
        Else
            mCol.Add(oChart, sKey, oBefore)
        End If
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to add a chart object to the collection by means of individual parameters.
    ''' </summary>
    ''' <param name="MedcinID">The Medcinid of the finding.</param>
    ''' <param name="Prefix">The prefix qualifier code.</param>
    ''' <param name="Modifier">The modifier qualifier code.</param>
    ''' <param name="Result">The result qualifier code.</param>
    ''' <param name="Status">The status qualifier code.</param>
    ''' <param name="Onset">The onset string.</param>
    ''' <param name="Duration">The duration string.</param>
    ''' <param name="Value">The data value of the observation.</param>
    ''' <param name="Unit">The unit of the data value.</param>
    ''' <param name="RangeScale">The range scale of the data value.</param>
    ''' <param name="RangeNormalLow">The normal low of the data value.</param>
    ''' <param name="RangeNormalHigh">The normal high of the data value.</param>
    ''' <param name="Referid">The referring provider docid.</param>
    ''' <param name="Flag">The flag.</param>
    ''' <param name="Note">The free text note attachment.</param>
    ''' <param name="Episode">The encoded episode string.</param>
    ''' <param name="QualifierLink">The qualifier link code.</param>
    ''' <param name="QualifierLinkSequence">The qualifier link sequence.</param>
    ''' <param name="rx">The associated MedPrescription object.</param>
    ''' <param name="Blobid">The id of the blob.</param>
    ''' <param name="SBlob">The annotation string.</param>
    ''' <param name="sKey">Optional. A unique String expression that specifies a key string 
    ''' that can be used instead of a positional index to access a member of the collection.</param>
    ''' <returns>A reference to the new MedChart object is returned.</returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Function Add(ByVal MedcinID As Integer, ByVal Prefix As String, ByVal Modifier As String, _
        ByVal Result As String, ByVal Status As String, ByVal Onset As String, _
        ByVal Duration As String, ByVal Value As String, ByVal Unit As String, _
        ByVal RangeScale As Short, ByVal RangeNormalLow As Double, ByVal RangeNormalHigh As Double, _
        ByVal Referid As Integer, ByVal Flag As Short, ByVal Note As String, ByVal Episode As String, _
        ByVal QualifierLink As String, ByVal QualifierLinkSequence As Integer, _
        ByVal rx As MedPrescription, ByVal Blobid As Integer, ByVal SBlob As String, Optional ByVal sKey As String = "") As MedChart
        'create a new object
        Dim objNewMember As New MedChart

        'set the properties passed into the method
        objNewMember.Medcinid = MedcinID
        objNewMember.Prefix = Prefix
        objNewMember.Modifier = Modifier
        objNewMember.Result = Result
        objNewMember.Status = Status
        objNewMember.Onset = Onset
        objNewMember.Duration = Duration
        objNewMember.Value = Value
        objNewMember.Unit = Unit
        objNewMember.RangeScale = RangeScale
        objNewMember.RangeNormalLow = RangeNormalLow
        objNewMember.RangeNormalHigh = RangeNormalHigh
        objNewMember.Referid = Referid
        objNewMember.Flag = Flag
        objNewMember.Note = Note
        objNewMember.Episode = Episode
        objNewMember.QualifierLink = QualifierLink
        objNewMember.QualifierLinkSequence = QualifierLinkSequence
        objNewMember.RxObj = rx
        objNewMember.Blobid = Blobid
        objNewMember.SBlob = SBlob
        If Len(sKey) = 0 Then
            mCol.Add(objNewMember)
        Else
            mCol.Add(objNewMember, sKey)
        End If

        'return the object created
        Add = objNewMember

    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to obtain one member of the collection by its index or key.
    ''' </summary>
    ''' <param name="IndexKey">The index or key of the item.</param>
    ''' <value>The MedChart object requested, or Nothing if not available.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Default Public ReadOnly Property Item(ByVal IndexKey As Object) As MedChart
        Get
            Try
                Return mCol.Item(IndexKey)
            Catch
                Return Nothing
            End Try
        End Get
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to obtain a count of the items in the collection.
    ''' </summary>
    ''' <value>The count of items.</value>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public ReadOnly Property Count() As Integer
        Get
            Count = mCol.Count()
        End Get
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This is used to recover the enumerator of the collection.
    ''' </summary>
    ''' <returns>The enumerator of the collection is returned.</returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    '''     [roy]   4/21/2008   Added Changed property
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        GetEnumerator = mCol.GetEnumerator
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' This allows you to remove a MedChart object from the collection.
    ''' </summary>
    ''' <param name="IndexKey">The index or key of the item.</param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[roy]	2/22/2005	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Sub Remove(ByVal IndexKey As Object)
        Try
            mCol.Remove(IndexKey)
        Catch

        End Try
    End Sub

    ''' <summary>
    ''' This creates a new MedCharts objct.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.New()
        mCol = New Collection
    End Sub

    Public Property Changed() As Boolean
        Get
            Return myChanged
        End Get
        Set(ByVal value As Boolean)
            myChanged = value
        End Set
    End Property

    ''' <summary>
    ''' This finalizes the object.
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overrides Sub Finalize()
        While (mCol.Count() > 1)
            mCol.Remove(1)
        End While
        mCol = Nothing
        MyBase.Finalize()
    End Sub
End Class