Module mFlowSheet
    Public Const RES_TIMEOUT As Integer = 101
    Public Const RES_NOIPID As Integer = 105
    Public Const STR_NOIPID As String = "No intelligent prompting data available for ~"
    Public Const RES_NOHISTORY As Integer = 103
    Public Const STR_NOHISTORY As String = "No historical data for that list"
    Public Const RES_EMPTYLIST As Integer = 109
    Public Const STR_EMPTYLIST As String = "The list contains no findings"
    Public Const RES_SPELLOK As Integer = 121
    Public Const STR_SPELLOK As String = "No misspelled words were found"

    Public Const ERR_STRING As String = "#error#"

    Public Const BMP_SQR_NEGBLK As Integer = 0
    Public Const BMP_SQR_NEGBLU As Integer = 1
    Public Const BMP_SQR_POSBLK As Integer = 2
    Public Const BMP_SQR_POSRED As Integer = 3
    Public Const BMP_RND_NEGBLK As Integer = 4
    Public Const BMP_RND_NEGBLU As Integer = 5
    Public Const BMP_RND_POSBLK As Integer = 6
    Public Const BMP_RND_POSRED As Integer = 7
    Public Const BMP_RDD_NEGBLK As Integer = 8
    Public Const BMP_RDD_NEGBLU As Integer = 9
    Public Const BMP_RDD_POSBLK As Integer = 10
    Public Const BMP_RDD_POSRED As Integer = 11
    Public Const BMP_ELIPSES As Integer = 12
    Public Const BMP_BTNNOTE As Integer = 13
    Public Const PREFIX_BLANK As String = " "
    Public Const RESULT_BLANK As String = " "
    Public Const RESULT_NEG As String = "N"
    Public Const RESULT_ABN As String = "A"

    Public Function ymd2mdy(ByVal sVal As String, Optional ByVal sShortYear As String = "") As String
        ' Function to convert Medcin date format (yyyymmddhhnn)
        ' to mm/dd/yyyy hh:nn format
        Dim strYear, strDate, strTime As String
        If (sVal.Length = 0) Then Return sVal
        Dim TheDate As String = ""
        Try
            Dim dlen As Short = sVal.Length
            If (dlen >= 8) Then dlen = 8
            strYear = sVal.Substring(0, 4)
            strDate = sVal.Substring(0, dlen)
            If (sVal.Length > 8) Then
                strTime = sVal.Substring(8)
            Else
                strTime = ""
            End If
            Select Case strDate.Length
                Case 6
                    TheDate = strDate.Substring(4, 2) & "/" & strYear
                Case 8
                    TheDate = strDate.Substring(4, 2) & "/" & strDate.Substring(6, 2) & "/" & strYear
                Case Else
                    TheDate = "01/01/" & strYear    ' Changed 07/30/2002
            End Select
            If (Len(sShortYear) > 0) Then
                TheDate = Format(CDate(TheDate), sShortYear)
            ElseIf (strTime.Length > 0) Then
                Dim hours As Integer
                hours = Val(strTime.Substring(0, 2))
                TheDate &= Format(hours, " 0#")
                If (strTime.Length > 3) Then
                    TheDate = TheDate & ":" & strTime.Substring(2, 2)   ' Minutes
                End If
                If (strTime.Length > 5) Then
                    TheDate = TheDate & ":" & strTime.Substring(4)      ' Seconds
                End If
            End If
        Catch ex As Exception
            TheDate = "--BAD DATE--"
        End Try
        Return TheDate
    End Function

    Friend Function AddDuration(ByVal dur As String) As String
        Dim Number As Object

        AddDuration = " lasting for " & dur
        Number = Val(dur)
        If (Number = 0) Then
            Exit Function
        End If
        AddDuration = " lasting for " & Number
    End Function

    Friend Function AddOnset(ByVal strOnset As String) As String
        ' Purpose: Takes a Medcin date format onset string and converts
        ' it to a presentation string for TranscribeChart.
        ' Medcin date format is yyyymmddhhnnss or a 'time period ago'
        ' expressed as 0yyy, 0000mm, 0000www, 000000dd, etc.
        Dim rc As String = ""
        Try
            If (strOnset.StartsWith("0")) Then
                rc = " " & ymd2ago(strOnset) & " ago"
            Else
                rc = " as of " & ymd2mdy(strOnset)
            End If
        Catch ex As Exception
            'PostError(ex)
        End Try
        Return rc
    End Function

    Friend Function ResolveResString(ByRef source As String, ByRef sText As String) As String
        Dim str As New System.Text.StringBuilder(source)
        str.Replace("~", sText)
        Return str.ToString
    End Function

    Friend Function RetrieveField(ByVal sData As String, ByVal sField As String) As String
        ' Function used to extract a single field from packed data of response item
        Dim intStartPos, intEndPos As Integer

        intStartPos = InStr(1, sData, sField)
        If (intStartPos > 0) Then
            intStartPos = intStartPos + 2
            intEndPos = InStr(intStartPos, sData, vbTab)
            If (intEndPos > 0) Then
                RetrieveField = Mid$(sData, intStartPos, intEndPos - intStartPos)
            Else
                RetrieveField = sData.Substring(intStartPos, sData.Length - intStartPos + 1)
            End If
        Else
            RetrieveField = ""
        End If
    End Function

    '
    ' Function to convert Medcin date format in time period
    ' syntax to string form of the time period
    '
    Public Function ymd2ago(ByVal strOnset As String) As String
        Dim intLen As Integer
        Dim strPrecision As String, strValue As String

        Select Case Len(strOnset)
            Case 4 : intLen = 4 : strPrecision = "year"
            Case 6 : intLen = 2 : strPrecision = "month"
            Case 7 : intLen = 3 : strPrecision = "week"
            Case 8 : intLen = 2 : strPrecision = "day"
            Case 10 : intLen = 2 : strPrecision = "hour"
            Case 12 : intLen = 2 : strPrecision = "minute"
            Case 14 : intLen = 2 : strPrecision = "second"
            Case Else
                ymd2ago = "#error#"
                Exit Function
        End Select
        strValue = strOnset.Substring(strOnset.Length - intLen)
        If (IsNumeric(strValue)) Then
            If (Val(strValue) > 0) Then strPrecision = strPrecision & "s"
            ymd2ago = CStr(Val(strValue)) & " " & strPrecision
        Else
            ymd2ago = "#error#"
        End If
    End Function

    'Friend Function ymd2mdy(ByVal sVal As String) As String
    '    ' Function to convert MedClient date format (yyyymmddhhnn)
    '    ' to mm/dd/yyyy hh:nn format
    '    Dim strYear, strDate, strTime As String
    '    Dim TheDate As String = ""
    '    If (sVal.Length = 0) Then Return sVal
    '    Try
    '        Dim dlen As Short = sVal.Length
    '        If (dlen >= 8) Then dlen = 8
    '        strYear = sVal.Substring(0, 4)
    '        strDate = sVal.Substring(0, dlen)
    '        If (sVal.Length > 8) Then
    '            strTime = sVal.Substring(8)
    '        Else
    '            strTime = ""
    '        End If
    '        Select Case strDate.Length
    '            Case 6
    '                TheDate = strDate.Substring(4, 2) & "/" & strYear
    '            Case 8
    '                TheDate = strDate.Substring(4, 2) & "/" & strDate.Substring(6, 2) & "/" & strYear
    '            Case Else
    '                TheDate = "01/01/" & strYear    ' Changed 07/30/2002
    '        End Select
    '        If (strTime.Length > 0) Then
    '            Dim hours As Integer = Val(strTime.Substring(0, 2))
    '            TheDate &= Format(hours, " 0#")
    '            If (strTime.Length > 3) Then
    '                TheDate = TheDate & ":" & strTime.Substring(2, 2)   ' Minutes
    '            Else
    '                TheDate = TheDate & ":00"   ' CDate needs minutes
    '            End If
    '            If (strTime.Length > 5) Then
    '                TheDate = TheDate & ":" & strTime.Substring(4)      ' Seconds
    '            End If
    '        End If
    '    Catch ex As Exception
    '        'MsgBox("ymd2mdy: " & ex.Message, MsgBoxStyle.Exclamation)
    '    End Try
    '    Return TheDate
    'End Function

End Module
